#!/usr/bin/env python3
import pickle,os,json,math,subprocess
import numpy as np
import pandas as pd
import tkinter as tk
from tkinter import ttk
from plateypus.setup.createTubeLayout import TubeLayoutPage 
from plateypus.setup.createPlateLayout import BlankSelectionPage

experimentParameters = {}
dataTypeDict = {'Cytokine':'cyt','Bulk/Single cell':'cell','Killing':'killing'}

class ExperimentSetupStartPage(tk.Frame):
    def __init__(self, master,fName,bPage):
        global folderName,backPage
        folderName = fName
        backPage = bPage
        tk.Frame.__init__(self, master)
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        v2 = tk.StringVar(value='inpt')
        rb2a = tk.Radiobutton(mainWindow, text="Enter experiment labels ",padx = 20, variable=v2, value='inpt')
        rb2b = tk.Radiobutton(mainWindow,text="Create experiment layout ",padx = 20, variable=v2, value='pl')
        rb2a.grid(row=0,column=0,sticky=tk.W)
        rb2b.grid(row=1,column=0,sticky=tk.W)
        
        #v3 = tk.StringVar(value='both')
        #l3 = tk.Label(mainWindow, text="for:     ")
        #l3.grid(row=0,column=1,rowspan=2,sticky=tk.EW)
        #rb3a = tk.Radiobutton(mainWindow, text="Cells",padx = 20, variable=v3, value='cell')
        #rb3b = tk.Radiobutton(mainWindow,text="Cytokines",padx = 20, variable=v3, value='cyt')
        #rb3c = tk.Radiobutton(mainWindow,text="Cells and Cytokines",padx = 20, variable=v3, value='both')
        #rb3a.grid(row=0,column=2,sticky=tk.W)
        #rb3b.grid(row=1,column=2,sticky=tk.W)
        #rb3c.grid(row=2,column=2,sticky=tk.W)
        
        labelList,cbList,cbVarList = [],[],[]
        for i,dataT in enumerate(list(dataTypeDict.keys())):
            l = tk.Label(mainWindow,text=dataT).grid(row=i,column=3,sticky=tk.W)
            v = tk.BooleanVar(value=False)
            cb = tk.Checkbutton(mainWindow, variable=v)
            cb.grid(row=i,column=1,sticky=tk.W)
            cbList.append(cb)
            cbVarList.append(v)
        
        def experimentLayout():
            for dataType in dataTypeList:
                if 'experimentParameters-'+folderName+'-'+dataType+'.json' in os.listdir('misc'):
                    experimentParameters = json.load(open('misc/experimentParameters-'+folderName+'-'+dataType+'.json','r'))
                    break
            if 'format' not in experimentParameters.keys():
                experimentParameters['format'] = 'plate'
            if experimentParameters['format'] == 'plate':
                plateDimensions = experimentParameters['overallPlateDimensions']
                levelLabelDict = experimentParameters['levelLabelDict']
                conditionLevelValues = levelLabelDict.copy()
                del levelLabelDict.copy()[list(levelLabelDict.keys())[-1]]
                levelValues = []
                for level in experimentParameters['levelLabelDict']:
                    levelValues.append(experimentParameters['levelLabelDict'][level])
                maxNumLevelValues = len(max(levelValues,key=len))
                levels = list(experimentParameters['levelLabelDict'].keys())
                if 'killing' in dataTypeList:
                    levels = levels[:-1]
                    levelValues = levelValues[:-1]
                master.switch_frame(BlankSelectionPage,folderName,levels,levelValues,maxNumLevelValues,experimentParameters['numPlates'],plateDimensions,dataTypeList,ExperimentSetupStartPage,bPage)
            #Tube mode
            else:
                fcsFiles = []
                
                if 'A1_cell.csv' not in os.listdir('inputData/bulkCSVFiles/'):
                    if len(os.listdir('inputData/fcsFiles')) > 1:
                        for fcsName in os.listdir('inputData/fcsFiles'):
                            if '.fcs' in fcsName:
                                fcsFiles.append(fcsName)
                        if len(fcsFiles) == 0:
                            fcsFiles = ['                 ']
                    elif 'sampleNameFile.xlsx' in os.listdir('misc'):
                        fcsFiles = list(pd.read_excel('misc/sampleNameFile.xlsx',header=0).iloc[:,0].values.ravel())
                else:
                    bulkStatFile = pd.read_csv('inputData/bulkCSVFiles/A1_cell.csv')
                    for row in range(bulkStatFile.shape[0]):
                        if bulkStatFile.iloc[row,0] not in ['Mean','SD']:
                            fcsFiles.append(bulkStatFile.iloc[row,0])

                master.switch_frame(TubeLayoutPage,folderName,experimentParameters['levelLabelDict'],len(fcsFiles),v3.get(),ExperimentSetupStartPage,bPage)
        
        def collectInput():
            global dataTypeList
            dataTypeList = [list(dataTypeDict.values())[i] for i,x in enumerate(cbVarList) if x.get()]
            if v2.get() == 'inpt':
                master.switch_frame(ExperimentFormatPage,folderName)
            elif v2.get() == 'pl':
                experimentLayout()

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInput()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class ExperimentFormatPage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        v1 = tk.StringVar()
        l1 = tk.Label(mainWindow,text='Collection Format: ')
        rb2a = tk.Radiobutton(mainWindow, text="Tube",padx = 20, variable=v1, value='tube')
        rb2b = tk.Radiobutton(mainWindow,text="Plate",padx = 20, variable=v1, value='plate')
        v1.set('tube')
        
        l1.grid(row=0,column=0)
        rb2a.grid(row=1,column=0)
        rb2b.grid(row=2,column=0)
        
        def collectInputs():
            if v1.get() == 'tube':
                experimentParameters['format'] = 'tube'
                master.switch_frame(TubeExperimentParameterPage,folderName)
            else:
                experimentParameters['format'] = 'plate'
                master.switch_frame(PlateExperimentParameterPage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(ExperimentSetupStartPage,folderName,backPage)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class TubeExperimentParameterPage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        

        l4 = tk.Label(mainWindow, text="How many samples are there in this experiment?")
        e0 = tk.Entry(mainWindow)
        l4.grid(row=0,column=0)
        e0.grid(row=0,column=1)
        
        l5 = tk.Label(mainWindow, text="How many unique levels are there in this experiment (excluding Time)?")
        e1 = tk.Entry(mainWindow)
        l5.grid(row=2,column=0)
        e1.grid(row=2,column=1)
        
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc/sampleNameFile.xlsx')
            columns = []
            for column in df:
                if column not in ['','FileName','Time']:
                    columns.append(column)
            e0.insert(tk.END,str(df.shape[0]))
            e1.insert(tk.END,str(len(columns)))

        def collectInputs():
            experimentParameters['numSamples'] = int(e0.get())
            experimentParameters['numAllLevels'] = int(e1.get())+1
            master.switch_frame(allLevelNamePage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(ExperimentFormatPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class PlateExperimentParameterPage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        
        Multiplexing_Options = {'96->384 Well Plate','Barcoding'}

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l7 = tk.Label(mainWindow, text="Multiplexing Options:")
        multiplex = tk.StringVar(value='None')
        plateMultiplexVar = tk.BooleanVar() 
        plateMultiplexVar2 = tk.BooleanVar() 
        barcodeMultiplexVar = tk.BooleanVar() 
        plateMultiplexCb = tk.Checkbutton(mainWindow,text='96->384 well',variable=plateMultiplexVar,onvalue=True,offvalue=False)
        plateMultiplexCbRev = tk.Checkbutton(mainWindow,text='384->96 well',variable=plateMultiplexVar2,onvalue=True,offvalue=False)
        plateMultiplexCbRev.config(state=tk.DISABLED)
        plateMultiplexCb.config(state=tk.NORMAL)
        barcodeMultiplexCb = tk.Checkbutton(mainWindow,text='Barcoding',variable=barcodeMultiplexVar,onvalue=True,offvalue=False)
        barcodingNumberLabel = tk.Label(mainWindow,text='# plates per barcoded plate') 
        barcodingNumberEntry = tk.Entry(mainWindow,width=4)

        v = tk.IntVar()
        v2 = tk.IntVar()
        v3 = tk.IntVar()
        v4 = tk.IntVar()
        
        def disable():
            if v3.get() == 384:
                plateMultiplexCb.config(state=tk.DISABLED)
                plateMultiplexCbRev.config(state=tk.NORMAL)
            else:
                plateMultiplexCbRev.config(state=tk.DISABLED)
                plateMultiplexCb.config(state=tk.NORMAL)

        l3 = tk.Label(mainWindow, text="""What format were the samples collected in?:""")
        rb3a = tk.Radiobutton(mainWindow, text="96 well plate",padx = 20, variable=v3, value=96,command=lambda: disable())
        rb3b = tk.Radiobutton(mainWindow,text="384 well plate",padx = 20, variable=v3, value=384,command=lambda: disable())
        l3.grid(row=0,column=0)
        rb3a.grid(row=0,column=1)
        rb3b.grid(row=0,column=2)
        v3.set(96)

        l4 = tk.Label(mainWindow, text="How many plates were used in the experiment?")
        e0 = tk.Entry(mainWindow)
        l4.grid(row=1,column=0)
        e0.grid(row=1,column=1)

        l6 = tk.Label(mainWindow, text="Enter the number of condition levels (excluding Time): ")
        e2 = tk.Entry(mainWindow)
        l6.grid(row=3,column=0)
        e2.grid(row=3,column=1)
        
        l7.grid(row=4,column=0)
        plateMultiplexCb.grid(row=4,column=1)
        plateMultiplexCbRev.grid(row=5,column=1)
        barcodeMultiplexCb.grid(row=4,column=2)
        barcodingNumberLabel.grid(row=4,column=3)
        barcodingNumberEntry.grid(row=4,column=4)
         
        def collectInputs():
            experimentParameters['numPlates'] = int(e0.get())
            
            #Incucyte datasets have time encoded in them already
            if 'killing' in dataTypeList:
                experimentParameters['numAllLevels'] = int(e2.get())
            else:
                experimentParameters['numAllLevels'] = int(e2.get())+1
            if v3.get() == 384:
                experimentParameters['overallPlateDimensions'] = [16,24]
            else:
                experimentParameters['overallPlateDimensions'] = [8,12]
            
            if (plateMultiplexVar.get() or plateMultiplexVar2.get()) and barcodeMultiplexVar.get():
                multiplexingOption = '96->384 well + Barcoding'
                numberOfBarcodes = barcodingNumberEntry.get()
            else:
                if plateMultiplexVar.get() or plateMultiplexVar2.get():
                    multiplexingOption = '96->384 well'
                    numberOfBarcodes = 1
                elif barcodeMultiplexVar.get():
                    multiplexingOption = 'Barcoding'
                    numberOfBarcodes = barcodingNumberEntry.get()
                else:
                    multiplexingOption = 'None'
                    numberOfBarcodes = 1
            if ',' in barcodingNumberEntry.get():
                numberOfBarcodes = list(map(int,numberOfBarcodes.split(',')))
            else:
                numberOfBarcodes = [int(numberOfBarcodes)]
            global globalMultiplexingVar
            globalMultiplexingVar = multiplexingOption
            experimentParameters['multiplexingOption'] = multiplexingOption
            if multiplexingOption == '96->384 well':
                master.switch_frame(MultiplexingPage,multiplexingOption)
            elif multiplexingOption == '96->384 well + Barcoding':
                master.switch_frame(BarcodingPage,multiplexingOption,numberOfBarcodes)
            elif multiplexingOption == 'Barcoding':
                master.switch_frame(BarcodingPage,multiplexingOption,numberOfBarcodes)
            else:
                master.switch_frame(allLevelNamePage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(ExperimentFormatPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class BarcodingPage(tk.Frame):
    def __init__(self, master,mpo,nb):
        tk.Frame.__init__(self, master)
        multiplexingOption = mpo
        BarcodingWindow = tk.Frame(self)
        BarcodingWindow.pack()
        numPl = experimentParameters['numPlates'] 
        
        numberOfBarcodes = math.ceil(np.log2(max(nb)))
        entryBarcodeLabelList = []
        for barcode in range(numberOfBarcodes):
            e = tk.Entry(BarcodingWindow,width=8)
            e.grid(row=0,column=2+barcode)
            e.insert(tk.END, 'Barcode '+str(barcode+1))
            entryBarcodeLabelList.append(e)
            #tk.Label(BarcodingWindow,text='Barcode '+str(barcode+1)).grid(row=0,column=2+barcode)
        
        if len(nb) == 1:
            numberOfBarcodedPlates = nb[0]
            if multiplexingOption == '96->384 well + Barcoding':
                if experimentParameters['overallPlateDimensions'][0] == 8: 
                    maxNumBarcoded = math.ceil(experimentParameters['numPlates'] / 4 / numberOfBarcodedPlates)
                    totalPlatesPerBarcodedPlate = 4*numberOfBarcodedPlates
                    plateSkip = 4
                #384->96
                else: 
                    maxNumBarcoded = math.ceil(4*experimentParameters['numPlates'] / numberOfBarcodedPlates)
                    totalPlatesPerBarcodedPlate = numberOfBarcodedPlates
                    plateSkip = 1 
            else:
                maxNumBarcoded = math.ceil(experimentParameters['numPlates'] / numberOfBarcodedPlates)
                totalPlatesPerBarcodedPlate = numberOfBarcodedPlates
                plateSkip = 1
            allIndividualPlateEntryList = []
            combinedPlateBarcodingVarList,combinedEntryList = [],[]
            for combinedPlateNum in range(maxNumBarcoded):
                combinedEntry = tk.Entry(BarcodingWindow,width=8)
                combinedEntry.insert(tk.END, 'A'+str(combinedPlateNum*totalPlatesPerBarcodedPlate+1)+'-'+str((combinedPlateNum+1)*totalPlatesPerBarcodedPlate))
                plateStart = combinedPlateNum*totalPlatesPerBarcodedPlate+1
                plateEnd = (combinedPlateNum+1)*totalPlatesPerBarcodedPlate
                allPlateIndices = list(range(plateStart,plateEnd+2,plateSkip))
                combinedEntry.grid(row=combinedPlateNum*numberOfBarcodedPlates+1,column=0)
                plateBarcodingVarList,plateEntryList = [],[]
                for plateNum in range(numberOfBarcodedPlates):
                    if plateNum == numberOfBarcodedPlates-1:
                        padvar = 10 
                    else:
                        padvar = 0
                    plateEntry = tk.Entry(BarcodingWindow,width=8)
                    if plateSkip == 4:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum])+'-'+str(allPlateIndices[plateNum+1]-1))
                    else:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum]))
                    plateEntry.grid(row=combinedPlateNum*numberOfBarcodedPlates+plateNum+1,column=1,pady=(0,padvar))
                    barcodingCbList,barcodingVarList = [],[]
                    for barcodeNum in range(numberOfBarcodes):
                        barcodeVar = tk.BooleanVar()
                        barcodeCb = tk.Checkbutton(BarcodingWindow,text='',variable=barcodeVar,onvalue=True,offvalue=False)
                        barcodeCb.grid(row=combinedPlateNum*numberOfBarcodedPlates+plateNum+1,column=2+barcodeNum,pady=(0,padvar))
                        barcodingCbList.append(barcodeCb)
                        barcodingVarList.append(barcodeVar)
                    plateBarcodingVarList.append(barcodingVarList)
                    plateEntryList.append(plateEntry)
                combinedPlateBarcodingVarList.append(plateBarcodingVarList)
                combinedEntryList.append(combinedEntry)
                allIndividualPlateEntryList.append(plateEntryList)
        else:
            if multiplexingOption == '96->384 well + Barcoding':
                if experimentParameters['overallPlateDimensions'][0] == 8: 
                    plateSkip = 4
                #384->96
                else: 
                    plateSkip = 1 
            else:
                plateSkip = 1
            allIndividualPlateEntryList = []
            combinedPlateBarcodingVarList,combinedEntryList = [],[]
            for elem,numberOfBarcodedPlates in enumerate(nb):
                summedPlateNums = sum(nb[:elem+1])
                plateEnd = plateSkip*summedPlateNums
                if elem == 0:
                    plateStart = 1
                    plateRow = 1
                else:
                    plateStart = sum(nb[:elem])*plateSkip + 1
                    plateRow = sum(nb[:elem]) + 1
                combinedEntry = tk.Entry(BarcodingWindow,width=8)
                combinedEntry.insert(tk.END, 'A'+str(plateStart)+'-'+str(plateEnd))
                combinedEntry.grid(row=plateRow,column=0)
                allPlateIndices = list(range(plateStart,plateEnd+2,plateSkip))
                plateBarcodingVarList,plateEntryList = [],[]
                for plateNum in range(numberOfBarcodedPlates):
                    if plateNum == numberOfBarcodedPlates-1 and plateNum != 0:
                        padvar = 10 
                    else:
                        padvar = 0
                    plateEntry = tk.Entry(BarcodingWindow,width=8)
                    if plateSkip == 4:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum])+'-'+str(allPlateIndices[plateNum+1]-1))
                    else:
                        plateEntry.insert(tk.END, 'A'+str(allPlateIndices[plateNum]))
                    plateEntry.grid(row=plateRow+plateNum,column=1,pady=(0,padvar))
                    barcodingCbList,barcodingVarList = [],[]
                    for barcodeNum in range(numberOfBarcodes):
                        barcodeVar = tk.BooleanVar()
                        barcodeCb = tk.Checkbutton(BarcodingWindow,text='',variable=barcodeVar,onvalue=True,offvalue=False)
                        barcodeCb.grid(row=plateRow+plateNum,column=2+barcodeNum,pady=(0,padvar))
                        barcodingCbList.append(barcodeCb)
                        barcodingVarList.append(barcodeVar)
                    plateBarcodingVarList.append(barcodingVarList)
                    plateEntryList.append(plateEntry)
                combinedPlateBarcodingVarList.append(plateBarcodingVarList)
                combinedEntryList.append(combinedEntry)
                allIndividualPlateEntryList.append(plateEntryList)

        l1 = tk.Label(BarcodingWindow,text='Barcoded Plate Name:')
        l2 = tk.Label(BarcodingWindow,text='Plate Name:')
        l1.grid(row=0,column=0)
        l2.grid(row=0,column=1)

        def collectInputs():
            barcodingDict = {}
            for combinedIndex,combinedEntry in enumerate(combinedEntryList):
                if combinedEntry.get() != '':
                    plateEntryList = allIndividualPlateEntryList[combinedIndex]
                    plateBarcodingDict = {}
                    for plateIndex,plateEntry in enumerate(plateEntryList):
                        if plateEntry.get() != '':
                            barcodingStringList = []
                            for barcodeIndex,barcodeVar in enumerate(combinedPlateBarcodingVarList[combinedIndex][plateIndex]):
                                if barcodeVar.get():
                                    barcodeSymbol = '+'
                                else:
                                    barcodeSymbol = '-'
                                barcodingStringList.append(entryBarcodeLabelList[barcodeIndex].get()+barcodeSymbol)
                            uncombinedPlateName = plateEntry.get()
                            if '_cyt' in plateEntry.get():
                                uncombinedPlateName = uncombinedPlateName[:-4]
                            elif '_cell' in plateEntry.get():
                                uncombinedPlateName = uncombinedPlateName[:-5]
                            plateBarcodingDict[uncombinedPlateName] = barcodingStringList
                    combinedPlateName = combinedEntry.get()
                    if '_cyt' in combinedEntry.get():
                        combinedPlateName = combinedPlateName[:-4]
                    elif '_cell' in combinedEntry.get():
                        combinedPlateName = combinedPlateName[:-5]
                    barcodingDict[combinedPlateName] = plateBarcodingDict
            experimentParameters['barcodingDict'] = barcodingDict
            if multiplexingOption == '96->384 well + Barcoding':
                master.switch_frame(MultiplexingPage,multiplexingOption)
            else:
                master.switch_frame(allLevelNamePage,folderName)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(PlateExperimentParameterPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class MultiplexingPage(tk.Frame):
    def __init__(self, master,mpo):
        tk.Frame.__init__(self, master)
        multiplexingOption = mpo
        Well384ConversionWindow = tk.Frame(self)
        Well384ConversionWindow.pack()
        
        reverse = False
        if multiplexingOption == '96->384 well + Barcoding':
            if experimentParameters['overallPlateDimensions'][0] == 16: 
                reverse = True

        if not reverse:
            maxNum384Plates = math.ceil(experimentParameters['numPlates'] / 4)
        else:
            maxNum384Plates = math.ceil(experimentParameters['numPlates'])
        Unpacking384PlateNameList = []
        Unpacking384List = []
        l1 = tk.Label(Well384ConversionWindow,text='Combined Plate Names: ')
        l1.grid(row=0,column=0)
        l2 = tk.Label(Well384ConversionWindow,text='Separated Plate Names: ')
        l2.grid(row=1,column=0)
        ttk.Separator().place(x=0, y=27, relwidth=1)
        ttk.Separator().place(x=157, y=0, relheight=0.65)
        for plateNum in range(maxNum384Plates):
            e1 = tk.Entry(Well384ConversionWindow,width=7)
            if not reverse:
                e1.insert(tk.END, 'A'+str(plateNum*4+1)+'-'+str((plateNum+1)*4))
            else:
                e1.insert(tk.END, 'A'+str(plateNum+1))
            e1.grid(row=0,column=plateNum*2+1)
            Unpacking384PlateNameList.append(e1)
            UnpackingWellPosList = []
            wellIndexList = [0,1,3,2]
            wellIndex = 0
            ttk.Separator().place(x=317+plateNum*160, y=0, relheight=0.65)
            for wellUnpackingRow in range(2):
                for wellUnpackingCol in range(2):
                    e2 = tk.Entry(Well384ConversionWindow,width=7)
                    e2.insert(tk.END, 'A'+str((plateNum*4)+wellIndexList[wellIndex]+1))
                    if wellUnpackingCol == 1:
                        padvar = 5
                    else:
                        padvar = 0
                    e2.grid(row=wellUnpackingRow+1,column=wellUnpackingCol+1+plateNum*2,padx=padvar)
                    UnpackingWellPosList.append(e2)
                    wellIndex+=1
            Unpacking384List.append(UnpackingWellPosList)

        def collectInputs():
            unpackingDict = {}
            for unpackingPlateNameEntry,unpackingWellPosList in zip(Unpacking384PlateNameList,Unpacking384List):
                tempList = []
                for unpackingEntry in unpackingWellPosList:
                    tempList.append(unpackingEntry.get())
                unpackingDict[unpackingPlateNameEntry.get()] = tempList
            experimentParameters['unpackingDict'] = unpackingDict
            master.switch_frame(allLevelNamePage,folderName)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(PlateExperimentParameterPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)
        
class allLevelNamePage(tk.Frame):
    def __init__(self, master,folderName):
        tk.Frame.__init__(self, master)
        numAllLevels = experimentParameters['numAllLevels']
            
        entryList1 = []
        entryList2 = []
        
        numericCheckBoxes = []
        numericCheckBoxVars = []

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        lt1 = tk.Label(mainWindow, text="Level Name").grid(row=0,column=1)
        lt2 = tk.Label(mainWindow, text="Number of Level Values").grid(row=0,column=2)
        lt3 = tk.Label(mainWindow, text="Numeric?").grid(row=0,column=3)
        
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc/sampleNameFile.xlsx')
            levels = []
            levelValueNums = []
            for level in df.columns:
                if level not in ['','FileName']:
                    levels.append(level)
                    levelValueNums.append(len(pd.unique(df[level])))
            j=0
        
        for conditionLevelNumber in range(1,numAllLevels+1):
            l1 = tk.Label(mainWindow, text="Condition "+str(conditionLevelNumber))
            e1 = tk.Entry(mainWindow)
            v = tk.IntVar()
            if conditionLevelNumber == 1 and 'killing' not in dataTypeList:
                #e1 = tk.Entry(mainWindow,state='readonly')
                e1.insert(tk.END, 'Time')
                e2 = tk.Entry(mainWindow)
                if 'sampleNameFile.xlsx' in os.listdir('misc'):
                    if 'Time' in df.columns:
                        e2.insert(tk.END,str(len(pd.unique(df['Time']))))
                    else: 
                        e2.insert(tk.END, '1')
                else:
                    e2.insert(tk.END, '1')
                v.set(1)
            else:
                e1 = tk.Entry(mainWindow)
                e2 = tk.Entry(mainWindow)
                v.set(0)
                if 'sampleNameFile.xlsx' in os.listdir('misc'):
                    e1.insert(tk.END, levels[j])
                    e2.insert(tk.END, str(levelValueNums[j]))
                    j+=1
                                    
            
            cb1 = tk.Checkbutton(mainWindow, text="",variable=v,onvalue=1,offvalue=0)
             
            l1.grid(row=conditionLevelNumber+1,column=0)
            e1.grid(row=conditionLevelNumber+1,column=1)
            e2.grid(row=conditionLevelNumber+1,column=2)
            entryList1.append(e1)
            entryList2.append(e2)
            cb1.grid(row=conditionLevelNumber+1,column=3)
            numericCheckBoxes.append(cb1)
            numericCheckBoxVars.append(v)
        
        def collectInputs():
            conditionNames = []
            numConditionLevelValues = []
            tiledLevels = []
            experimentParameters['columnVariableName'] = 'Time' 
            numericlevels = []
            for allLevelNumber in range(numAllLevels):
                #Remove column variable from condition name list
                if entryList1[allLevelNumber].get() == 'Time':
                    experimentParameters['numColumnLevelValues'] = int(entryList2[allLevelNumber].get())
                else:
                    conditionNames.append(str(entryList1[allLevelNumber].get()))
                    numConditionLevelValues.append(int(entryList2[allLevelNumber].get()))
                numericlevels.append(numericCheckBoxVars[allLevelNumber].get() == 1)
            if 'killing' in dataTypeList:
                experimentParameters['numColumnLevelValues'] = 0 
                experimentParameters['numConditionLevels'] = numAllLevels
            else:
                experimentParameters['numConditionLevels'] = numAllLevels - 1

            experimentParameters['conditionLevelNames'] = conditionNames
            experimentParameters['allLevelNames'] = [experimentParameters['columnVariableName']]+conditionNames
            experimentParameters['numConditionLevelValues'] = numConditionLevelValues
            experimentParameters['numericLevels'] = numericlevels
            experimentParameters[''] = tiledLevels
            master.switch_frame(columnLevelValuesPage,folderName)

        def backCommand():
            if experimentParameters['format'] == 'tube':
                master.switch_frame(TubeExperimentParameterPage,folderName)
            else:
                master.switch_frame(PlateExperimentParameterPage,folderName)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=numAllLevels+1,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: backCommand()).grid(row=numAllLevels+1,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=numAllLevels+1,column=2)

class columnLevelValuesPage(tk.Frame):
    def __init__(self, master,folderName):
        numColumnLevelValues = experimentParameters['numColumnLevelValues']
        
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        lt = tk.Label(mainWindow,text=experimentParameters['columnVariableName']+':').grid(row=0,column=0)
        col_wrap = 12
        for col in range(1,numColumnLevelValues+1):
            lt1 = tk.Label(mainWindow, text='Level Value '+str(col),width=10).grid(row=int((col-1)/col_wrap)*2,column=((col-1)%col_wrap)+1)
        entryList = []
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc/sampleNameFile.xlsx')
            if 'Time' in df.columns:
                times = list(map(float,list(pd.unique(df['Time']))))
                sortedTimes = list(map(str,sorted(times)))
                j=0

        for columnLevelValueNumber in range(numColumnLevelValues):
            e1 = tk.Entry(mainWindow,width=10)
            e1.grid(row=2*int(columnLevelValueNumber/col_wrap)+1,column=(columnLevelValueNumber%col_wrap)+1)
            if 'sampleNameFile.xlsx' in os.listdir('misc'):
                if 'Time' in df.columns:
                    e1.insert(tk.END,sortedTimes[j]) 
                    j+=1
            entryList.append(e1)

        def collectInputs():
            columnLevelValues = []
            for entry in entryList:
                columnLevelValues.append(float(entry.get()))
            experimentParameters['columnLevelValues'] = columnLevelValues
            master.switch_frame(conditionLevelValuesPage,folderName)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=2*int(numColumnLevelValues/col_wrap)+2,column=5)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(allLevelNamePage,folderName)).grid(row=2*int(numColumnLevelValues/col_wrap)+2,column=6)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=2*int(numColumnLevelValues/col_wrap)+2,column=7)

class LinkedEntryButton(tk.Button):
    def __init__(self,parent,linkedentries,**kwargs):
        tk.Button.__init__(self,parent,**kwargs)
        self.parent = parent
        self.linkedentries = linkedentries

    def fillNumerically(self):
        for i,entry in enumerate(self.linkedentries):
            entry.insert(tk.END,str(i+1))

class conditionLevelValuesPage(tk.Frame):
    def __init__(self, master,folderName):
        numConditionLevels = experimentParameters['numConditionLevels']
        maxLevelValues = max(experimentParameters['numConditionLevelValues'])
        
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        entryWrap = 16
        fullEntryList = []
        blist = []

        labelList = experimentParameters['conditionLevelNames']
        rowNum = 0
        if 'sampleNameFile.xlsx' in os.listdir('misc'):
            df = pd.read_excel('misc/sampleNameFile.xlsx')
        for conditionLevelNumber in range(numConditionLevels):
            if 'sampleNameFile.xlsx' in os.listdir('misc'):
                j = 0
            currentLabel = labelList[conditionLevelNumber]
            initialRowNum = rowNum
            l1 = tk.Label(mainWindow, text='Level values for \"'+currentLabel+'\":').grid(row=rowNum,column=1,sticky=tk.W) 
            levelEntryList = []
            rowNum-=1
            for col in range(1,maxLevelValues+1):
                if 'sampleNameFile.xlsx' in os.listdir('misc'):
                    for column in df.columns:
                        if column == currentLabel:
                            filledInLevelValues = list(pd.unique(df[column]))
                if (col-1)%entryWrap == 0:
                    rowNum+=1
                if col < experimentParameters['numConditionLevelValues'][conditionLevelNumber]+1:
                    e1 = tk.Entry(mainWindow,width=8)
                    e1.grid(row=rowNum,column=(col-1)%entryWrap+2,sticky=tk.W)
                    if 'sampleNameFile.xlsx' in os.listdir('misc'):
                        e1.insert(tk.END,filledInLevelValues[j])
                        j+=1
                    levelEntryList.append(e1)
            b = LinkedEntryButton(mainWindow,levelEntryList,text='Fill Values Numerically')
            b.configure(command=b.fillNumerically)
            b.grid(row=initialRowNum,column=0,sticky=tk.W)
            blist.append(b)
            fullEntryList.append(levelEntryList)
            rowNum+=1

        def collectInputs():
            conditionLevels = {}
            for lvlentrylist,i in zip(fullEntryList,range(numConditionLevels)):
                tempLevels = []
                for entry in lvlentrylist:
                    tempLevels.append(str(entry.get()))
                conditionLevels[experimentParameters['conditionLevelNames'][i]] = tempLevels
            experimentParameters['conditionLevelValues'] = conditionLevels.copy()
            experimentParameters['allLevelValues'] = conditionLevels.copy()
            experimentParameters['allLevelValues'][experimentParameters['columnVariableName']] = experimentParameters['columnLevelValues']
            shortenedExperimentParameters = {}
            for keyToKeep in ['format','numPlates','overallPlateDimensions','barcodingDict','unpackingDict']:
                if keyToKeep in list(experimentParameters.keys()):
                    shortenedExperimentParameters[keyToKeep] = experimentParameters[keyToKeep]
            shortenedExperimentParameters['levelLabelDict'] = experimentParameters['allLevelValues']
            for dataType in dataTypeList:
                with open('misc/experimentParameters-'+folderName+'-'+dataType+'.json', 'w') as fp:
                    json.dump(shortenedExperimentParameters, fp)
            #{"A1-12": {"A1-4": ["Barcode 1-", "Barcode 2-"], "A5-8": ["Barcode 1+", "Barcode 2-"], "A9-12": ["Barcode 1-", "Barcode 2+"]}}
            #{"A1-4": ["A1", "A2", "A4", "A3"], "A5-8": ["A5", "A6", "A8", "A7"], "A9-12": ["A9", "A10", "A12", "A11"]}
            if 'cell' in dataTypeList and experimentParameters['format'] == 'plate':
                #Both
                if '+' in globalMultiplexingVar:
                    for plate in experimentParameters['barcodingDict']:
                        if plate not in os.listdir('inputData/singleCellCSVFiles/'):
                            subprocess.run(['mkdir','inputData/singleCellCSVFiles/'+plate])
                else:
                    if 'Barcoding' in globalMultiplexingVar:
                        for plate in experimentParameters['barcodingDict']:
                            if plate not in os.listdir('inputData/singleCellCSVFiles/'):
                                subprocess.run(['mkdir','inputData/singleCellCSVFiles/'+plate])
                    elif '96' in globalMultiplexingVar:
                        for plate in experimentParameters['unpackingDict']:
                            if plate not in os.listdir('inputData/singleCellCSVFiles/'):
                                subprocess.run(['mkdir','inputData/singleCellCSVFiles/'+plate])
                    else:
                        for plateNum in range(1,experimentParameters['numPlates']+1):
                            if 'A'+str(plateNum) not in os.listdir('inputData/singleCellCSVFiles/'):
                                subprocess.run(['mkdir','inputData/singleCellCSVFiles/A'+str(plateNum)])
            master.switch_frame(ExperimentSetupStartPage,folderName,backPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=numConditionLevels+1,column=int(maxLevelValues/2))
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(columnLevelValuesPage,folderName)).grid(row=numConditionLevels+1,column=int(maxLevelValues/2)+1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=numConditionLevels+1,column=int(maxLevelValues/2)+2)



#!/usr/bin/env python3 
import json,pickle,math,matplotlib,sys,glob
from sys import platform as sys_pf
if sys_pf == 'darwin':
    import matplotlib
    matplotlib.use("TkAgg")
import numpy as np
import pandas as pd
import seaborn as sns
import tkinter as tk
from matplotlib import pyplot as plt
from scipy.optimize import curve_fit
from itertools import groupby
from plateypus.dataprocessing.miscFunctions import Hill,InverseHill,r_squared,cleanUpFlowjoCSV
idx = pd.IndexSlice

#Th1/2/17 Mouse  BD Biosciences CBA Kit Cytokines
bdMouseThKitDict = {'IFNg':17200,'IL-2':17200,'IL-4':14000,'IL-6':21900,'IL-10':18900,'IL-17A':15500,'TNFa':17500}
#Inflammatory Mouse  BD Biosciences CBA Kit Cytokines
bdMouseInfKitDict = {'IFNg':17200,'IL-6':21900,'IL-10':18900,'TNFa':17500,'IL-12p70':57480,'MCP-1':16000}
#Inflammatory Human BD Biosciences CBA Kit Cytokines
bdHumanInfKitDict = {'IL-12p70':70000,'IL-1B':17300,'IL-8':8904}
#Chemokine Human BD Biosciences CBA Kit Cytokines
bdHumanChemokineKitDict = {'IP-10':86000,'MCP-1':13000,'MIG':11700,'RANTES':7809,'IL-8':8904}
#Legendplex 13-plex mouse th kit
legendPlexMouseThKitDict = {'IFNg':15652,'IL-5':26200,'TNFa':25896,'IL-2':17231,'IL-6':21709,'IL-4':13500,'IL-10':20641,'IL-9':14300,'IL-17A':14978,'IL-17F':14900,'IL-21':14400,'IL-22':16800,'IL-13':12300}
#Legendplex 13-plex mouse macrophage kit
legendPlexMouseMacrophageKitDict = {'G-CSF':18800,'GM-CSF':14000,'IL-12':75000,'IL-12p40':40000,'IL-18':18000,'IL-1b':17500,'IL-23':55000,'IL-34':39000,'IL-7':17000,'KC':11000,'MCP-1':12000,'TARC':8000,'TGFb':25000}

#Combine all cytokine bead kit molar masses:
listOfKitDicts = [legendPlexMouseThKitDict,bdMouseThKitDict,bdHumanInfKitDict,bdMouseInfKitDict,bdHumanChemokineKitDict,legendPlexMouseMacrophageKitDict]
completeCytokineMWDict = {}
for dictName in listOfKitDicts:
    completeCytokineMWDict.update(dictName)
"""
#Standard BD Biosciences Th1/2/17 Kit Cytokines
listOfCytokines1=['IFNg','IL-2','IL-4','IL-6','IL-10','IL-17A','TNFa']
MWofCytokines1=[17200,17200,14000,21900,18900,15500,17500] #g/mol, correct one

#Standard BD Biosciences Inflamamatory Kit Cytokines
listOfCytokines2=['IL-12p70','IL-1B','IL-8']
MWofCytokines2=[70000,17300,8904] #g/mol, correct one

#Standard Mouse Th Legendplex CBA Kit Cytokines
#Up to IL-4 is A (lower FSC and SSC), after that is B
legendPlexListOfCytokines1=['IFNg','IL-5','TNFa','IL-2','IL-6','IL-4','IL-10','IL-9','IL-17A','IL-17F','IL-21','IL-22','IL-13']
"""

class CalibrationParameterPage(tk.Frame):
    def __init__(self, master,folderName,expNum,ex_data,shp,bPage):
        tk.Frame.__init__(self, master)
        
        secondaryhomepage = shp
        
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10) 
        
        l1 = tk.Label(mainWindow,text='Number of CBA calibration samples: ').grid(row=0,column=0)
        t1 = tk.Entry(mainWindow)
        t1.grid(row=1,column=0,sticky=tk.W)

        l2 = tk.Label(mainWindow,text='Volume of initial CBA calibration solution: ').grid(row=0,column=1)
        t2 = tk.Entry(mainWindow)
        t2.grid(row=1,column=1,sticky=tk.W)
         
        def collectInputs():
            numCalibrationSamples = int(t1.get())
            initialStandardVolume = float(t2.get())
            calibrationParameterDict = {'Number': numCalibrationSamples,'Volume': initialStandardVolume}
            with open('misc/CBAcalibrationParameters-'+folderName+'.json','w') as f:
                json.dump(calibrationParameterDict,f)
            master.switch_frame(secondaryhomepage,folderName,expNum,ex_data,bPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

def parseCytokineCSVHeaders(columns):
    #,Beads/IFNg | Geometric Mean (YG586-A),Beads/IL-2 | Geometric Mean (YG586-A),Beads/IL-4 | Geometric Mean (YG586-A),Beads/IL-6 | Geometric Mean (YG586-A),Beads/IL-10 | Geometric Mean (YG586-A),Beads/IL-17A | Geometric Mean (YG586-A),Beads/TNFa | Geometric Mean (YG586-A),
    newMultiIndexList = []
    for column in columns[1:-1]:
        populationNameVsStatisticSplit = column.split(' | ')
        if 'Barcode' not in populationNameVsStatisticSplit[0]:
            cytokine = populationNameVsStatisticSplit[0].split('/')[-1]
        else:
            cytokine = populationNameVsStatisticSplit[0].split('/')[-2]
        newMultiIndexList.append([cytokine])
    return newMultiIndexList

def performCommaCheck(fileName):
    with open('inputData/bulkCSVFiles/'+fileName, 'r') as istr:
        with open('inputData/bulkCSVFiles/'+fileName, 'w') as ostr:
            for line in istr:
                if line[-1] != ',':
                    line = line.rstrip('\n') + ','
                    print(line, file=ostr)

#Create Calibration Curves, obtain LODs (limits of detection) of experiment
def calibrateExperiment(folderName,secondPath,concUnit,concUnitPrefix,numberOfCalibrationSamples,initialStandardVolume):
    #Get cytokine calibration curve data
    tempExperimentParameters = {'overallPlateDimensions':[8,12]}
    calibrationFileNames = glob.glob('inputData/bulkCSVFiles/Calibration*')
    calibrationNames = []
    kitNames = []
    for calibrationFileName in calibrationFileNames:
        #performCommaCheck(calibrationFileName.split('/')[-1])
        newName = calibrationFileName.split('.')[0].split('_')[0].split('/')[-1]
        kitNames.append(newName)
    sortedData,sortedFiles = cleanUpFlowjoCSV(kitNames,folderName,'cyt',tempExperimentParameters)
    for i,newName in enumerate(kitNames):
        if '-' in newName:
            newName2 = newName.split('-')[1]
        else:
            newName2 = newName
        kitNames[i] = newName2
    rsquaredList = []
    concLODList = []
    fittingParametersList = []
    cbaStandardsMFIList = []
    cbaPlotPointsMFIList = []
    cbaStandardsConcentrationList = []
    cbaPlotPointsConcentrationList = []

    numberOfPlotPoints = 101
    xaxistitle = 'Concentration of Cytokine Standards Standards ('+concUnitPrefix+')' 
    yaxistitle = 'GeoMFI'
        
    allCytokinesHaveMWInDict = True 
    for calibration in sortedData:
        cytokines = parseCytokineCSVHeaders(calibration.columns)
        for cytokine in cytokines:
            if cytokine[0] not in completeCytokineMWDict:
                allCytokinesHaveMWInDict = False

    for calibration in sortedData:
        data = np.array(calibration.values[:,1:-1],dtype=float)
        cytokines = parseCytokineCSVHeaders(calibration.columns)
        fittingParameters = np.zeros((data.shape[1],4))
        concLOD = np.zeros((data.shape[1],4))
        #Initial concentration all cytokine standards is given by CBA kit manual as 5000 pGg/mL: when standards are diluted in 2mL
        conc = 5000 #pg/mL
        serialDilutionFactor = 2 #1:serialDilutionFactor dilution between each standard well
        #Smaller initial dilution (0.5mL instead of 2mL for example) increase the initial concentration of the first calibration sample
        initialConc = (conc*1e-12) /((initialStandardVolume*1e-3)/2) #g/L (pg/mL * 1e-12 g/pg)/(1e-3 L/mL)
        #Calibration samples are always diluted by a factor of serialdilutionFactor (so with 12 calibration samples, the last sample is (serialDilutionFactor^-11) the concentration of the first, which is pure standard (2^0)
        cbaStandardsConcentrations = np.flipud(initialConc*np.power(serialDilutionFactor,np.linspace(-numberOfCalibrationSamples+1,0,numberOfCalibrationSamples)))
        #More x values along the above concentration bounds are sampled to use to construct calibration curve. Plot points are extended slightly at high range to allow visualization of upper LOD not accessible with experimental dilution
        cbaStandardsConcentrationsPlotPoints = np.flipud(initialConc*np.power(2,np.linspace(-numberOfCalibrationSamples+1,4,numberOfPlotPoints)))
        
        cbaStandardsConcentrationMatrix = np.zeros([len(cytokines),cbaStandardsConcentrations.shape[0]]) 
        cbaStandardsConcentrationPlotPointsMatrix = np.zeros([len(cytokines),cbaStandardsConcentrationsPlotPoints.shape[0]])
        cbaStandardsMFIMatrix = np.zeros([len(cytokines),cbaStandardsConcentrations.shape[0]]) 
        cbaStandardsMFIPlotPointsMatrix = np.zeros([len(cytokines),cbaStandardsConcentrationsPlotPoints.shape[0]])
        color_list = sns.color_palette(sns.color_palette(),len(cytokines))
        for i,cytokineList in enumerate(cytokines):
            cytokine = cytokineList[0]
            #amplitude bounded from range/2 to range*2, EC50 bounded from minimum to maximum standard concentration tested, Hill coefficient bounded from 0 to 2, Background bounded from 0 to minimum GFI*2
            lowerCurveFitBounds = [(np.max(data[:,i])-np.min(data[:,i]))/2,np.min(cbaStandardsConcentrations),0,0]
            upperCurveFitBounds = [(np.max(data[:,i])-np.min(data[:,i]))*2, np.max(cbaStandardsConcentrations), 2,np.min(data[:,i])*2]
            #use scipy curve fit to determine best hill equation fit for data, searching within the bounds given above
            popt,pcov = curve_fit(Hill, cbaStandardsConcentrations,np.log10(data[:,i]),sigma=np.log10(data[:,i]),bounds=(lowerCurveFitBounds,upperCurveFitBounds))
            rsquared = round(r_squared(cbaStandardsConcentrations,np.log10(data[:,i]),Hill,popt),3)
            rsquaredList.append(rsquared)
            for j in range(len(popt)):  
                #Convert just ec50 value to desired units (nM,uM etc) if cytokine has a molar mass in dict
                if j == 1 and allCytokinesHaveMWInDict:
                    fittingParameters[i,j] = np.multiply(popt[j],(concUnit/completeCytokineMWDict[cytokine]))
                #other values in 4 parameter logistic equation are tied to intensity y-value, which doesn't change, or are the hill coefficient, which is completely separate, so parameters are kept the same
                else:
                    fittingParameters[i,j] = popt[j]
            
            #Convert x values of experimental data points and curve fit points to desired units (nM,uM,etc.)
            if allCytokinesHaveMWInDict:
                cbaStandardsConcentrationMatrix[i,:] = np.multiply(cbaStandardsConcentrations,(concUnit/completeCytokineMWDict[cytokine]))
                cbaStandardsConcentrationPlotPointsMatrix[i,:] = np.multiply(cbaStandardsConcentrationsPlotPoints,(concUnit/completeCytokineMWDict[cytokine]))
            else:
                cbaStandardsConcentrationMatrix[i,:] = cbaStandardsConcentrations
                cbaStandardsConcentrationPlotPointsMatrix[i,:] = cbaStandardsConcentrationsPlotPoints
            cbaStandardsMFIMatrix[i,:] = data[:,i]
            cbaStandardsMFIPlotPointsMatrix[i,:] = np.power(10,Hill(cbaStandardsConcentrationPlotPointsMatrix[i,:],*fittingParameters[i,:]))
            #Plot on log-log scale the experimental points and the curve fit line with previously determined curve fitting parameters
            #plt.loglog(cbaStandardsConcentrations,data[:,i],'o',color=color_list[i,:],label=listOfCytokines[i])
            #plt.loglog(cbaStandardsConcentrationsPlotPoints,np.power(10,Hill(convertedCBAStandardsPlotPoints,*fittingParameters[i,:])))
            #'_fit; R2 = '+str(rsquared)

            #Get LOD for each cytokine calibration curve (aka the linear range of the calibration curve)
            backgroundGFI = fittingParameters[i,3]
            amplitudeGFI = fittingParameters[i,0]
            
            #Approximate LOD by determining concentration values at LOD% and 1-LOD% (3% and 97%) of curve. Must be used on log10(curve), as calibration curve is plotted in logscale
            LODpercent = 0.03
            #LOD% more than background GFI used for lower LOD GFI
            lowerGFILOD = math.log10(10**((1+LODpercent)*math.log10(backgroundGFI)))
            #LOD% less than maximum GFI (Background + amplitude) used for upper LOD GFI
            upperGFILOD = math.log10(10**((1-LODpercent)*math.log10(amplitudeGFI+backgroundGFI)))
            #Log10(upper/lowerGFILOD) converted back into normal GFI by 10 to its power, then fed into inverse hill equation with current cytokine fitting parameters to obtain corresponding concentration values
            lowerConcLOD = InverseHill(lowerGFILOD,fittingParameters[i,:])
            upperConcLOD = InverseHill(upperGFILOD,fittingParameters[i,:])
            #Create dict with keys as cytokines, values as GFI/conc LODs
            concLOD[i,:] = np.array([10**lowerGFILOD,10**upperGFILOD,lowerConcLOD,upperConcLOD])
        flattenedMatrix = cbaStandardsMFIMatrix.flatten()
        reshapedMatrix = np.reshape(flattenedMatrix,(numberOfCalibrationSamples,len(cytokines)),order='F')
        flattenedMatrix2 = cbaStandardsMFIPlotPointsMatrix.flatten()
        reshapedMatrix2 = np.reshape(flattenedMatrix2,(numberOfPlotPoints,len(cytokines)),order='F')
        flattenedMatrix3 = cbaStandardsConcentrationMatrix.flatten()
        reshapedMatrix3 = np.reshape(flattenedMatrix3,(numberOfCalibrationSamples,len(cytokines)),order='F')
        flattenedMatrix4 = cbaStandardsConcentrationPlotPointsMatrix.flatten()
        reshapedMatrix4 = np.reshape(flattenedMatrix4,(numberOfPlotPoints,len(cytokines)),order='F')
        realCytokineList = []
        for cytokine in cytokines:
            realCytokineList.append(cytokine[0])
        dataValsList = []
        plotPointsList = []
        for j in range(1,numberOfCalibrationSamples+1):
            dataValsList.append([j])
        for j in range(1,numberOfPlotPoints+1):
            plotPointsList.append([j])
        dataValsIndex = pd.MultiIndex.from_tuples(dataValsList,names=['Standard'])
        plotPointsIndex = pd.MultiIndex.from_tuples(plotPointsList,names=['Standard'])
        currentCBAStandardsMFIDf = pd.DataFrame(reshapedMatrix,index=dataValsIndex,columns=realCytokineList)
        currentCBAPlotPointsMFIDf = pd.DataFrame(reshapedMatrix2,index=plotPointsIndex,columns=realCytokineList)
        currentCBAStandardsConcentrationDf = pd.DataFrame(reshapedMatrix3,index=dataValsIndex,columns=realCytokineList)
        currentCBAPlotPointsConcentrationDf = pd.DataFrame(reshapedMatrix4,index=plotPointsIndex,columns=realCytokineList)
       
        currentCBAStandardsMFIDf.columns.name = 'Cytokine' 
        currentCBAPlotPointsMFIDf.columns.name = 'Cytokine'
        currentCBAStandardsConcentrationDf.columns.name = 'Cytokine' 
        currentCBAPlotPointsConcentrationDf.columns.name = 'Cytokine'
        mic1 = pd.MultiIndex.from_tuples(cytokines,names=['Cytokine'])
        fittingParametersDf = pd.DataFrame(fittingParameters,index=mic1,columns=['Amplitude','EC50','HillCoeff','Background'])
        mic2 = pd.MultiIndex.from_tuples([['MFI','Lower'],['MFI','Upper'],['Concentration','Lower'],['Concentration','Upper']])
        concLODDf = pd.DataFrame(concLOD,index=mic1,columns=mic2)

        concLODList.append(concLODDf)
        fittingParametersList.append(fittingParametersDf)
        cbaStandardsMFIList.append(currentCBAStandardsMFIDf)
        cbaPlotPointsMFIList.append(currentCBAPlotPointsMFIDf)
        cbaStandardsConcentrationList.append(currentCBAStandardsConcentrationDf)
        cbaPlotPointsConcentrationList.append(currentCBAPlotPointsConcentrationDf)

    #fullFittingParametersDf = pd.concat(fittingParametersList,keys=kitNames,names=['Kit Name'])
    #fullConcLODDf = pd.concat(concLODList,keys=kitNames,names=['Kit Name'])
    fullFittingParametersDf = pd.concat(fittingParametersList)
    fullConcLODDf = pd.concat(concLODList)
    
    fullCBAStandardsMFIDf = pd.concat(cbaStandardsMFIList,keys=kitNames,names=['Kit Name'],axis=1)
    fullCBAPlotPointsMFIDf = pd.concat(cbaPlotPointsMFIList,keys=kitNames,names=['Kit Name'],axis=1)
    fullCBAStandardsConcentrationDf = pd.concat(cbaStandardsConcentrationList,keys=kitNames,names=['Kit Name'],axis=1)
    fullCBAPlotPointsConcentrationDf = pd.concat(cbaPlotPointsConcentrationList,keys=kitNames,names=['Kit Name'],axis=1)
    fullCBAStandardsList = [fullCBAStandardsMFIDf.stack().stack(),fullCBAStandardsConcentrationDf.stack().stack()]
    fullCBAPlotPointsList = [fullCBAPlotPointsMFIDf.stack().stack(),fullCBAPlotPointsConcentrationDf.stack().stack()]
    fullCBAStandardsDf = pd.concat(fullCBAStandardsList,axis=1,keys=[yaxistitle,xaxistitle])
    fullCBAPlotPointsDf = pd.concat(fullCBAPlotPointsList,axis=1,keys=[yaxistitle,xaxistitle])
     
    plottingPointsDf = fullCBAPlotPointsDf.reset_index()
    plottingStandardsDf = fullCBAStandardsDf.reset_index()
    
    numCyt = len(pd.unique(plottingPointsDf['Cytokine']))
    if numCyt <= 10:
        fullpalette = sns.color_palette(sns.color_palette(),numCyt)
    else:
        fullpalette = sns.color_palette('hls',numCyt)
    g = sns.relplot(data=plottingPointsDf,x=xaxistitle,y=yaxistitle,hue='Cytokine',col='Kit Name',kind='line',col_order=pd.unique(plottingPointsDf['Kit Name']),hue_order=pd.unique(plottingPointsDf['Cytokine']),height=7,palette=fullpalette)
    #Plot vertical lines at lower and upper concentration limits of detection
    colorDict = {}
    for j,cytokine in enumerate(pd.unique(plottingPointsDf['Cytokine'])):
        colorDict[cytokine] = fullpalette[j]
    for axis,kitName in zip(g.axes.flat,pd.unique(plottingPointsDf['Kit Name'])):
        currentpalette = []
        for cytokine in pd.unique(plottingStandardsDf.query("`Kit Name` == @kitName")['Cytokine']):
            currentColor = colorDict[cytokine]
            currentpalette.append(currentColor)
            cytokineLODValues = fullConcLODDf.loc[cytokine,:]['Concentration']
            axis.axvline(x=cytokineLODValues['Lower'].values,color=currentColor,linestyle=':')
            axis.axvline(x=cytokineLODValues['Upper'].values,color=currentColor,linestyle=':')
        g2 = sns.scatterplot(data=plottingStandardsDf[plottingStandardsDf['Kit Name'] == kitName],x=xaxistitle,y=yaxistitle,hue='Cytokine',ax=axis,legend=False,palette=currentpalette)
        axis.set_xscale('log')
        axis.set_yscale('log')
    plt.savefig('plots/calibrationCurves-'+folderName+'-'+concUnitPrefix+'.png',bbox_inches='tight')
    #Save fitting parameters and LOD for curve fit for each cytokine
    with open('misc/fittingParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "wb") as f:
        pickle.dump(fullFittingParametersDf, f)
    with open('misc/LODParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "wb") as f:
        pickle.dump(fullConcLODDf, f)

def createCytokineDataFrame(folderName,finalDataFrame,concUnitPrefix):
         
        columnName = finalDataFrame.columns.name
        with open('outputData/pickleFiles/cytokineGFIPickleFile-'+folderName+'.pkl', "wb") as f:
            pickle.dump(finalDataFrame, f)
        
        fittingParameters = pickle.load(open('misc/fittingParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "rb"))
        LODParameters = pickle.load(open('misc/LODParameters-'+folderName+'-'+concUnitPrefix+'.pkl', "rb"))
        #Begin converting GFI dataframe into corresponding concentration dataframe
        concentrationList = []
        #Step through dataframe one cytokine at a time
        for cytokine in pd.unique(finalDataFrame.index.get_level_values(0)):
            #Retrieve LODs for current cytokine (from constructed calibration curve)
            lowerGFILOD = LODParameters.loc[cytokine,idx['MFI','Lower']].values[0]
            upperGFILOD = LODParameters.loc[cytokine,idx['MFI','Upper']].values[0] 
            lowerConcLOD = LODParameters.loc[cytokine,idx['Concentration','Lower']].values[0]
            upperConcLOD = LODParameters.loc[cytokine,idx['Concentration','Upper']].values[0] 
            smallConcentrationMatrix = np.zeros(finalDataFrame.loc[cytokine].shape)
            #Loop through every value in current cytokine's portion of the dataframe
            for i in range(0,finalDataFrame.loc[cytokine].values.shape[0]):
                for j in range(0,finalDataFrame.loc[cytokine].values.shape[1]):
                    currentGFIval = finalDataFrame.loc[cytokine].values[i,j]
                    if currentGFIval > upperGFILOD: #If intensity is greater than upper GFI LOD
                        currentConcVal = upperConcLOD #Concentration is equal to upper concentration LOD
                    elif currentGFIval <= upperGFILOD and currentGFIval >= lowerGFILOD: #if intensity is between upper and lower GFI LODs
                        currentConcVal = InverseHill(np.log10(currentGFIval),*fittingParameters.loc[cytokine,:].values) #Use previous hill fit parameters for the cytokine to obtain concentration
                    else: #If intensity is less than background GFI LOD
                        currentConcVal = lowerConcLOD #Concentration is equal to lower concentration LOD
                    smallConcentrationMatrix[i,j] = currentConcVal
            concentrationList.append(smallConcentrationMatrix)

        concentrationMatrix = np.vstack(concentrationList)
        finalDataFrameConcentration = pd.DataFrame(concentrationMatrix,index=finalDataFrame.index,columns=finalDataFrame.columns)
        finalDataFrameConcentration.columns.name = columnName 
        return finalDataFrameConcentration



#!/usr/bin/env python3
import pickle,os,subprocess
import tkinter as tk
import tkinter.ttk
import pandas as pd
from plateypus.setup.experimentCreationGUI import NewExperimentWindow,NewProjectWindow,RemoveProjectWindow
from plateypus.setup.experimentSetupGUI import ExperimentSetupStartPage
from plateypus.dataprocessing.miscFunctions import setMaxWidth
from plateypus.dataprocessing.dataProcessingGUI import DataProcessingStartPage
from plateypus.plotting.plottingGUI import PlotExperimentWindow 
import plateypus
from PIL import Image,ImageTk
from importlib_metadata import version

#Root class; handles frame switching in gui
class MainApp(tk.Tk):
    def __init__(self):
        self.root = tk.Tk.__init__(self)

        self.title('plateypus '+version('plateypus'))
        self._frame = None
        self.homedirectory = '/'.join(os.path.abspath(plateypus.__file__).split('/')[:-1])
        if self.homedirectory[-1] != '/':
            self.homedirectory+='/'
        print('plateypus location: '+self.homedirectory)
        self.switch_frame(ExperimentSelectionPage)

    def switch_frame(self, frame_class,*args):
        """Destroys current frame and replaces it with a new one."""
        new_frame = frame_class(self,*args)
        if self._frame is not None:
            self._frame.destroy()
        self._frame = new_frame
        self._frame.pack()

#Top level actions for experiments
class ExperimentSelectionPage(tk.Frame):
    def __init__(self,master):
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
        
        #load = Image.open("/Users/acharsr/Documents/plateypus/docs/plateypusLogo.png")
        load = Image.open(master.homedirectory+"plateypusLogo.png")
        width, height = load.size
        SCALEFACTOR = 0.4
        load = load.resize((int(SCALEFACTOR*width), int(SCALEFACTOR*height)), Image.ANTIALIAS)
        render = ImageTk.PhotoImage(load)
        img = tk.Label(mainWindow, image=render)
        img.image = render
        img.grid(row=0,column=0,columnspan=3)
        v = tk.StringVar(value='slt')
        rb1c = tk.Button(mainWindow, text="Associate new experiment with project",padx = 20, command=lambda:master.switch_frame(NewExperimentWindow,ExperimentSelectionPage))
        rb1c.grid(row=3,column=0,sticky=tk.EW,columnspan=3,pady=1)
        rb1a = tk.Button(mainWindow, text="Create new project",padx = 20, command=lambda:master.switch_frame(NewProjectWindow,ExperimentSelectionPage))
        rb1b = tk.Button(mainWindow, text="Remove project",padx = 20, command=lambda:master.switch_frame(RemoveProjectWindow,ExperimentSelectionPage))
        rb1a.grid(row=1,column=0,sticky=tk.EW,columnspan=3,pady=(20,1))
        rb1b.grid(row=2,column=0,sticky=tk.EW,columnspan=3,pady=1)
        
        expSelectionFrame = tk.Frame(mainWindow,borderwidth=1,relief=tk.SOLID)
        expSelectionFrame.grid(row=4,column=0,sticky=tk.EW,columnspan=3,pady=(10,0))
        rb1d = tk.Button(expSelectionFrame,text="Work on existing experiment ",padx = 20, command=lambda: collectInput())
        
        def getUpdateData(event):
            projectName = self.projectMenu.get()
            pathName = self.pathDict[projectName]
            experiments = []
            for experimentName in os.listdir(pathName+projectName):
                if '.DS' not in experimentName:
                    experiments.append(experimentName)
            experiments = sorted(experiments)[::-1]
            self.experimentMenu['values'] = experiments
            if len(experiments) == 1:
                self.experimentMenu.set(self.experimentMenu['values'][0])
            self.experimentMenu['width'] = len(max(experiments,key=len))
        
        if 'misc' not in os.listdir(master.homedirectory):
            subprocess.run(['mkdir',master.homedirectory+'misc'])
        if 'pathDict.pkl' not in os.listdir(master.homedirectory+'misc'):
            self.pathDict = {}
        else:
            self.pathDict = pickle.load(open(master.homedirectory+'misc/pathDict.pkl','rb'))
        projects = list(self.pathDict.keys())
        self.projectMenu = tkinter.ttk.Combobox(expSelectionFrame,values = projects)
        if len(self.pathDict) > 0:
            self.projectMenu['width'] = len(max(projects,key=len))
        tk.Label(expSelectionFrame,text='Project name: ').pack()
        self.projectMenu.pack()
        self.projectMenu.bind('<<ComboboxSelected>>', getUpdateData)

        self.experimentMenu = tkinter.ttk.Combobox(expSelectionFrame)
        tk.Label(expSelectionFrame,text='Experiment name: ').pack()
        self.experimentMenu.pack()
        rb1d.pack(expand=True,fill=tk.BOTH,pady=(5,0))

        def collectInput():
            projectName = self.projectMenu.get()
            pathName = self.pathDict[projectName]
            selectedExperiment = self.experimentMenu.get()
            os.chdir(pathName+projectName+'/'+selectedExperiment)
            master.switch_frame(ExperimentActionWindow,selectedExperiment)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,padx=10,pady=(50,10))
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)

class ExperimentActionWindow(tk.Frame):
    def __init__(self,master,selectedExperiment):
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
        
        l1 = tk.Label(mainWindow,text='Choose an action:')
        v = tk.StringVar(value='plt')
        rb1a = tk.Radiobutton(mainWindow, text="Setup experiment",padx = 20, variable=v, value='se')
        rb1b = tk.Radiobutton(mainWindow,text="Process experiment",padx = 20, variable=v, value='pd')
        rb1c = tk.Radiobutton(mainWindow,text="Plot experiment",padx = 20, variable=v, value='plt')
        l1.grid(row=0,column=0)
        rb1a.grid(row=1,column=0,sticky=tk.W)
        rb1b.grid(row=2,column=0,sticky=tk.W)
        rb1c.grid(row=3,column=0,sticky=tk.W)
        
        def collectInput():
            action = v.get()
            if action == 'se':
                master.switch_frame(ExperimentSetupStartPage,selectedExperiment,ExperimentActionWindow)
            elif action == 'pd':
                master.switch_frame(DataProcessingStartPage,selectedExperiment,32,[],ExperimentActionWindow)
            elif action == 'plt':
                master.switch_frame(PlotExperimentWindow,selectedExperiment,ExperimentActionWindow)
        
        def backCommand():
            os.chdir(master.homedirectory)
            master.switch_frame(ExperimentSelectionPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,padx=10,pady=10)
        tk.Button(buttonWindow, text="OK",command=lambda: collectInput()).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Back",command=lambda: backCommand()).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)

if __name__== "__main__":
    app = MainApp()
    app.mainloop()



#!/usr/bin/env python3 
import pickle,os,sys,string,subprocess
import numpy as np
import pandas as pd
import tkinter as tk
from plateypus.dataprocessing.miscFunctions import reindexDataFrame,printProgressBar,extractValues,reorderDfByInputOrder,returnSpecificExtensionFiles
idx = pd.IndexSlice

plateRowLetters = list(string.ascii_uppercase)[:16]
plateColumnNumbers = list(range(1,25))

def produceSingleCellHeaders(cellTypes):
    newMultiIndexList = []
    for cellType in cellTypes:
        newMultiIndexList.append([cellType])
    return newMultiIndexList

def grabCellTypeList(experimentParameters):
    path = 'inputData/singleCellCSVFiles/'
    if experimentParameters['format'] == 'plate':
        nrows = experimentParameters['overallPlateDimensions'][0]
        ncols = experimentParameters['overallPlateDimensions'][1]
        if 'unpackingDict' in experimentParameters.keys():
            orderingList = [str(x).zfill(3) for x in range(1,385)]
        else:
            orderingList = [str(x).zfill(3) for x in range(1,nrows*ncols+1)]

        #Plate case
        #Walk through a plate, grab each unique cell type
        cellTypeList = []
        folder = ''
        for fileName in os.listdir(path):
            if '.DS' not in fileName:
                if 'barcodingDict' in experimentParameters:
                    if fileName not in experimentParameters['barcodingDict']:
                        folder = fileName
                        break
                else:
                    folder = fileName
                    break

        for fileName in os.listdir(path+folder+'/'):
            if '.DS' not in fileName:
                splitFile = fileName.split('_')
                parsingVal = ''
                for split in splitFile:
                    if split in orderingList:
                        parsingVal = split
                parsingPose = fileName.rindex(parsingVal)+4
                cellType = fileName[parsingPose:].split('.')[0].replace('/','\/')
                if cellType not in cellTypeList:
                    cellTypeList.append(cellType)
    else:
        cellTypeList = []
        sampleNameDf = pd.read_pickle('misc/tubeLayout-'+os.getcwd().split('/')[-1]+'-cell.pkl')
        fullSampleFileName = sampleNameDf.iloc[0,0]
        dotIndex = fullSampleFileName.rfind('.')
        sampleFileName = fullSampleFileName[:dotIndex]
        for fileName in os.listdir(path):
            if '.DS' not in fileName and sampleFileName in fileName:
                splitFile = fileName.split(sampleFileName[1:]+'_')
                cellType = splitFile[1].split('.')[0]
                if cellType not in cellTypeList:
                    cellTypeList.append(cellType)

    return cellTypeList

def createTubeSingleCellDataFrame(folderName,experimentParameters,fileNameDf):
    
    path = 'inputData/singleCellCSVFiles/'
    
    fileNameDf = fileNameDf.stack().to_frame('fileName')

    #Grab common prefix for files
    fullFileName = ''
    for fileName in os.listdir(path):
        if '.DS' not in fileName:
            fullFileName = fileName
            break
    prefix = fullFileName.split('_')[0]

    #Remove extraneous decorations in marker names
    newColumns = []
    fcsDf = pd.read_csv(path+fullFileName,header=0)
    for column in fcsDf.columns:
        if '::' in column:
            column = column.split(' :: ')[1]
        #CyTOF
        if column[0].isdigit():
            column = column.split('_')[1]
        newColumns.append(column)

    cellTypeList = grabCellTypeList(experimentParameters)
    
    completeDfList = []
    for cellType in cellTypeList:
        for row in range(fileNameDf.shape[0]):
            fullFileName = fileNameDf.iloc[row,0]
            dotIndex = fullFileName.rfind('.')
            fileName = fullFileName[:dotIndex]
            trueFileName = prefix+'_'+fileName+'_'+cellType+'.csv'
            levelValues = list(fileNameDf.iloc[row,:].name) 
            fcsDf = pd.read_csv(path+trueFileName,header=0)
            eventNumber = fcsDf.shape[0]
            eventList = range(1,eventNumber+1)
            allLevelValues = []
            for event in eventList:
                allLevelValues.append([cellType]+levelValues+[event])
            newMultiIndex = pd.MultiIndex.from_tuples(allLevelValues,names=['CellType']+list(fileNameDf.index.names)+['Event'])
            newDf = pd.DataFrame(fcsDf.values,index=newMultiIndex,columns=newColumns)
            completeDfList.append(newDf)
            printProgressBar(row + 1, fileNameDf.shape[0], prefix = ' Concatenating samples:', suffix = 'Complete', length = 50)

    completeDataFrame = pd.concat(completeDfList)
    completeDataFrame.columns.name = 'Marker'

    #Remove extraneous markers (namely -h parameters)
    columnsToKeep = []
    for col,column in enumerate(completeDataFrame.columns):
        if '-H' not in column and 'Time' not in column and '-W' not in column and column != 'GFP' and column != 'LiveDead':
            columnsToKeep.append(col)
    completeDataFrame = completeDataFrame.iloc[:,columnsToKeep]

    completeDataFrame.to_hdf('outputData/pickleFiles/initialSingleCellDf-channel-'+folderName+'.h5', key='df', mode='w')
    print(completeDataFrame)

def debarcodeSingleCellData(experimentParameters):
    barcodingDict = experimentParameters['barcodingDict']
    for barcodedPlateName in barcodingDict:
        inputPath = 'inputData/singleCellCSVFiles/'+barcodedPlateName+'/'
        for debarcodedPlateName in barcodingDict[barcodedPlateName]:
            #Create folder if not there; if there, delete folder before proceeding (avoids overlap)
            outputPath = 'inputData/singleCellCSVFiles/'+debarcodedPlateName+'/'
            if debarcodedPlateName in os.listdir('inputData/singleCellCSVFiles/'):
                subprocess.run(['rm','-r',outputPath[:-1]])
            subprocess.run(['mkdir',outputPath[:-1]])
            
            barcodes = barcodingDict[barcodedPlateName][debarcodedPlateName]
            #Search through all files in barcoded plate folder, move to each debarcoded plate folder
            for k,fileName in enumerate([x for x in os.listdir(inputPath) if '.DS' not in x]):
                allBarcodesInFileName = True
                for barcode in barcodes:
                    if barcode not in fileName:
                        allBarcodesInFileName = False
                        break
                if allBarcodesInFileName:
                    #Copy file to debarcoded plate folder
                    subprocess.run(['cp',inputPath+fileName,outputPath+fileName])
                    #Rename file; if __ exists, use suffix after __ as cell population name. otherwise no suffix
                    #In either case, remove barcode suffix
                    if '__' in fileName:
                        population = fileName.split('.')[0].split('__')[1]
                        newFileName = '_'.join(fileName.split('.')[0].split('__')[0].split('_')[:6]+[population])+'.csv'
                    else:
                        newFileName = '_'.join(fileName.split('.')[0].split('_')[:6]+['allCells'])+'.csv'
                    subprocess.run(['mv',outputPath+fileName,outputPath+newFileName])
                printProgressBar(k + 1, len([x for x in os.listdir(inputPath) if '.DS' not in x]), prefix = ' Debarcoding '+barcodedPlateName+' to '+debarcodedPlateName+':', suffix = 'Complete', length = 50)

#If multiplexing option chosen
def demultiplexSingleCellData(experimentParameters):
    unpackingDict = experimentParameters['unpackingDict']
    unpackingPositionDict = {(0,0):0,(0,1):1,(1,0):2,(1,1):3}
    cellTypeList = grabCellTypeList(experimentParameters)

    #Currently in format A1; A2, B1; B2
    #Need to change to format A1: B1, A2, B2
    plateRowLetters = string.ascii_uppercase[:16]
    plateColumnNumbers = list(range(1,25))

    wellPlateRowLetters = string.ascii_uppercase[:8]
    wellPlateColumnNumbers = list(range(1,13))

    #Unpacking
    if experimentParameters['overallPlateDimensions'][0] == 8:
        #"multiplexingOption": "96->384 well", "unpackingDict": {"A1-2_B1-2": ["A1", "A2", "B2", "B1"]
        #Create appropriate folders in each population
        for combinedPlateName in list(unpackingDict.keys()):
            for unpackedPlateName in unpackingDict[combinedPlateName]:
                if unpackedPlateName != '':
                    if unpackedPlateName not in returnSpecificExtensionFiles('inputData/singleCellCSVFiles/','',False):
                        subprocess.run(['mkdir','inputData/singleCellCSVFiles/'+unpackedPlateName])
        fileNameDict = {}
        combinedPlateNames = list(unpackingDict.keys())
        for combinedPlateName in combinedPlateNames:
            #scale_Specimen_001_P9_P09_369_TCells.csv
            allFileNames = returnSpecificExtensionFiles('inputData/singleCellCSVFiles/'+combinedPlateName,'',False)
            unpackedPlateNames = unpackingDict[combinedPlateName]
            for k,fileName in enumerate(allFileNames):
                sampleID = fileName.split('_')[3]
                currentRowLetter = sampleID[0]
                currentColumnNumber = int(sampleID[1:])
                #Get index of current row and column position
                currentRowLetterIndex = plateRowLetters.index(currentRowLetter)
                currentColumnNumberIndex = plateColumnNumbers.index(currentColumnNumber)
                #Demultiplex sample ids 
                wellPlateRowLetter = wellPlateRowLetters[int(currentRowLetterIndex/2)]
                wellPlateColumnNumber = wellPlateColumnNumbers[int(currentColumnNumberIndex/2)]
                newSampleID = str(wellPlateRowLetter)+str(wellPlateColumnNumber)
                newSampleID2 = str(wellPlateRowLetter)+str(wellPlateColumnNumber).zfill(2)
                newFileName = '_'.join(['_'.join(fileName.split('_')[:3]),newSampleID,newSampleID2,'_'.join(fileName.split('_')[-2:])])
                unpackedPlateIndex = unpackingPositionDict[(currentRowLetterIndex%2,currentColumnNumberIndex%2)]
                unpackedFolder = unpackedPlateNames[unpackedPlateIndex]
                trueFileName = newFileName
                #print(fileName+'->'+newFileName)
                fileNameDict['_'.join(trueFileName.split('_')[1:-1])] = '_'.join(fileName.split('_')[1:-1])
                completeNewFileName = unpackedFolder+'/'+trueFileName
                subprocess.run(['cp','inputData/singleCellCSVFiles/'+combinedPlateName+'/'+fileName,'inputData/singleCellCSVFiles/'+completeNewFileName])
                printProgressBar(k + 1, len(allFileNames), prefix = ' Unpacking '+combinedPlateName+':', suffix = 'Complete', length = 50)
    #Repacking
    else:
        wellIDConversionDict = {}
        for i,plateLetter in enumerate(plateRowLetters):
            for j,plateNumber in enumerate(plateColumnNumbers):
                wellIDConversionDict[plateLetter+str(plateNumber)] = plateRowLetters[int(i/2)]+str(plateColumnNumbers[int(j/2)])
        plateIDConversionDict = {}
        for multiplexedPlateName in experimentParameters['unpackingDict']:
            for i,multiplexedWellPos in enumerate(experimentParameters['unpackingDict'][multiplexedPlateName]):
                wellIDs = []
                if multiplexedWellPos != '':
                    if i < 2:
                        j = 0
                    else:
                        j = 1
                    for rowIndex in range(j,len(plateRowLetters),2):
                        for colIndex in range(i%2,len(plateColumnNumbers),2):
                            wellID = plateRowLetters[rowIndex]+str(plateColumnNumbers[colIndex])
                            wellIDs.append(wellID)
                plateIDConversionDict[multiplexedWellPos] = wellIDs
        fileNameDict = {}
        combinedPlateNames = list(unpackingDict.keys())
        for combinedPlateName in combinedPlateNames:
            if combinedPlateName in os.listdir('inputData/singleCellCSVFiles/'):
                suffix = '-unpacked'
                subprocess.run(['mv','inputData/singleCellCSVFiles/'+combinedPlateName,'inputData/singleCellCSVFiles/'+combinedPlateName+'-unpacked'])
            else:
                suffix = ''
            subprocess.run(['mkdir','inputData/singleCellCSVFiles/'+combinedPlateName])
            for plateName in unpackingDict[combinedPlateName]:
                if plateName == combinedPlateName:
                    suffix = '-unpacked'
                else:
                    suffix = ''
                specificWellIDConversionDict = {}
                for well in plateIDConversionDict[plateName]:
                    specificWellIDConversionDict[well] = wellIDConversionDict[well]
                invertedWellIDConversionDict = {v: k for k, v in specificWellIDConversionDict.items()}
                #scale_Specimen_001_P9_P09_369_TCells.csv
                allFileNames = returnSpecificExtensionFiles('inputData/singleCellCSVFiles/'+plateName+suffix,'',False)
                unpackedPlateNames = unpackingDict[combinedPlateName]
                for k,fileName in enumerate(allFileNames):
                    sampleID = fileName.split('_')[3]
                    newSampleID =  invertedWellIDConversionDict[sampleID]
                    newSampleID2 = newSampleID[0]+newSampleID[1:].zfill(3)
                    newFileName = '_'.join(['_'.join(fileName.split('_')[:3]),newSampleID,newSampleID2,'_'.join(fileName.split('_')[-2:])])
                    fileNameDict['_'.join(newFileName.split('_')[1:-1])] = '_'.join(fileName.split('_')[1:-1])
                    subprocess.run(['cp','inputData/singleCellCSVFiles/'+plateName+suffix+'/'+fileName,'inputData/singleCellCSVFiles/'+combinedPlateName+'/'+newFileName])
                    printProgressBar(k + 1, len(allFileNames), prefix = ' Repacking '+plateName+':', suffix = 'Complete', length = 50)

    with open('misc/fileNameDict.pkl','wb') as f:
        pickle.dump(fileNameDict,f)

def createPlateSingleCellDataFrame(folderName,experimentParameters,levelLayout,useBlankWells):
    
    path = 'inputData/singleCellCSVFiles/'
    #Change CyTEK file names to match BD file names
    for folder in os.listdir(path):
        if '.DS' not in folder:
            for fileName in os.listdir(path+folder):
                if '.DS' not in fileName:
                    if ' Well' in fileName:
                        newFileNameComponents = fileName.split(' Well')
                        newFileName = newFileNameComponents[0].split('_')[0]+'_Specimen_001_'+newFileNameComponents[0].split('_')[1]+'_'+newFileNameComponents[0].split('_')[1][0]+newFileNameComponents[0].split('_')[1][1:].zfill(3)+newFileNameComponents[1]
                        #print('singleCellCSVFiles/'+folder+'/'+fileName+'->'+'singleCellCSVFiles/'+folder+'/'+newFileName)
                        subprocess.run(['mv',path+folder+'/'+fileName,path+folder+'/'+newFileName])
    
    if 'barcodingDict' in experimentParameters:
        debarcodeSingleCellData(experimentParameters)
    if 'unpackingDict' in experimentParameters:
        demultiplexSingleCellData(experimentParameters)
    
    nrows = experimentParameters['overallPlateDimensions'][0]
    ncols = experimentParameters['overallPlateDimensions'][1]

    if 'unpackingDict' in experimentParameters.keys():
        orderingList = [str(x).zfill(3) for x in range(1,385)]
    else:
        orderingList = [str(x).zfill(3) for x in range(1,nrows*ncols+1)]

    completeKeyMatrix = np.dstack(list(levelLayout['keys'].values()))
    unraveledKeyMatrix = np.reshape(completeKeyMatrix,(completeKeyMatrix.shape[0]*completeKeyMatrix.shape[1],completeKeyMatrix.shape[2]))
    unraveledBlankMatrix = levelLayout['blank'].ravel()
    #print(unraveledBlankMatrix)

    sampleIndex = pd.MultiIndex.from_arrays([levelLayout['plateID'].ravel(),levelLayout['wellID'].ravel()],names=['Plate','Well'])
    sampleKeyDf = pd.DataFrame(unraveledKeyMatrix,index=sampleIndex,columns=list(experimentParameters['levelLabelDict'].keys()))
    sampleDf = sampleKeyDf.copy()
    rowsToKeep = []
    
    #print(sampleKeyDf.iloc[:,3].values.ravel())
    for row in range(sampleDf.shape[0]):
        for col in range(sampleDf.shape[1]):
            level = list(experimentParameters['levelLabelDict'].keys())[col]
            levelValueIndex = sampleKeyDf.iloc[row,col]
            if unraveledBlankMatrix[row] == -1:
                levelValue = experimentParameters['levelLabelDict'][level][levelValueIndex]
                sampleDf.iloc[row,col] = levelValue
            else:
                sampleDf.iloc[row,col] = 'Blank'
    #Drop blanks
    sampleDf = sampleDf.query("Time != 'Blank'")
    sampleDf.to_excel('outputData/excelFiles/fcsLabelingKey.xlsx')
    
    cellTypeList = grabCellTypeList(experimentParameters)

    newSampleDfList = []
    for cellType in cellTypeList:
        #Create dict that relates sample file locations to plate/wellIDs
        orderValDict = {}
        for plateName in list(np.unique(levelLayout['plateID'])):
            orderValDict2 = {}
            if 'DS' not in plateName:
                for fileName in os.listdir(path+plateName+'/'):
                    if 'DS' not in fileName:
                        splitFile = fileName.split('_')
                        parsingVal = ''
                        for split in splitFile:
                            if split in orderingList:
                                parsingVal = split
                        parsingPose = fileName.rindex(parsingVal)
                        currentCellType = fileName[parsingPose+4:].split('.')[0]
                        if currentCellType == cellType:
                            orderVal = fileName[parsingPose:parsingPose+3]
                            wellID = fileName[:parsingPose-1].split('_')[-2]
                            orderValDict2[wellID] = path+plateName+'/'+fileName
            orderValDict[plateName] = orderValDict2
        sampleTupleList,sampleList = [],[]
        for row in range(sampleDf.shape[0]):
            sampleID = list(sampleDf.iloc[row,:].name)
            plateID = sampleID[0]
            wellID = sampleID[1]
            sampleFileName = orderValDict[plateID][wellID]
            sampleList.append(sampleFileName)
            if type(cellType) != list:
                if type(cellType) == tuple:
                    cellType = list(cellType)
                else:
                    cellType = [cellType]
            sampleTuple = cellType+sampleDf.loc[idx[plateID,wellID],:].values.tolist()
            sampleTupleList.append(sampleTuple)
        sampleMI = pd.MultiIndex.from_tuples(sampleTupleList,names=['CellType']+list(sampleDf.columns))
        newSampleDf = pd.DataFrame(sampleList,index=sampleMI,columns=['fileName'])
        newSampleDfList.append(newSampleDf)
    
    fileNameDf = pd.concat(newSampleDfList)

    bothBool = False
    fullFileName = fileNameDf.iloc[0,0]
    fcsDf = pd.read_csv(fullFileName,header=0)
    for column in fcsDf.columns:
        if '::' in column:
            bothBool = True
    
    completeDfList = []
    for row in range(fileNameDf.shape[0]):
        fullFileName = fileNameDf.iloc[row,0]
        levelValues = list(fileNameDf.iloc[row,:].name) 
        fcsDf = pd.read_csv(fullFileName,header=0)
        eventNumber = fcsDf.shape[0]
        if eventNumber == 0:
            if not useBlankWells:
                tk.messagebox.showerror("Error", "Filename:\n"+fullFileName+"\nhas no events. Please re-export this file and try again.")
                sys.exit(0)
            else:
                newMatrix = np.zeros([1,fcsDf.shape[1]])
                fcsDf = pd.DataFrame(newMatrix,columns=fcsDf.columns)
                eventNumber+=1
        eventList = range(1,eventNumber+1)
        allLevelValues = []
        for event in eventList:
            allLevelValues.append(levelValues+[event])
        newMultiIndex = pd.MultiIndex.from_tuples(allLevelValues,names=list(fileNameDf.index.names)+['Event'])
        if bothBool:
            newColumns = [x.split(' :: ')[1] if '::' in x else x for x in fcsDf.columns]
        else:
            newColumns = fcsDf.columns
        newDf = pd.DataFrame(fcsDf.values,index=newMultiIndex,columns=newColumns)
        completeDfList.append(newDf)
        printProgressBar(row + 1, fileNameDf.shape[0], prefix = ' Concatenating samples:', suffix = 'Complete', length = 50)
    
    #Remove all unnecessary folders (only keep input/output)
    foldersToKeep = []
    for folder in [x for x in os.listdir('inputData/singleCellCSVFiles/') if '.DS' not in x]:
        #Keep all folders
        if 'unpackingDict' in experimentParameters and 'barcodingDict' in experimentParameters and experimentParameters['overallPlateDimensions'][0] == 16:
            if folder in experimentParameters['barcodingDict'] or folder in experimentParameters['unpackingDict']:
                foldersToKeep.append(folder)
        else:
            foldersToKeep.append(folder)

    for folder in [x for x in os.listdir('inputData/singleCellCSVFiles/') if '.DS' not in x]:
        if folder not in foldersToKeep:
            subprocess.run(['rm','-r','inputData/singleCellCSVfiles/'+folder])

    completeDataFrame = pd.concat(completeDfList)
    completeDataFrame.columns.name = 'Marker'

    #Remove extraneous markers (namely -h/-w parameters)
    columnsToKeep = []
    for col,column in enumerate(completeDataFrame.columns):
        #if '-H' not in column and 'Time' not in column and '-W' not in column:
        if '-H' not in column and 'Time' not in column and '-W' not in column and column != 'GFP' and 'Live' not in column:
            columnsToKeep.append(col)
    completeDataFrame = completeDataFrame.iloc[:,columnsToKeep]

    completeDataFrame.to_hdf('outputData/pickleFiles/initialSingleCellDf-channel-'+folderName+'.h5', key='df', mode='w')
    print(completeDataFrame)



#!/usr/bin/env python3 
import json,pickle,os
import numpy as np
import pandas as pd
import tkinter as tk
from plateypus.dataprocessing import initialDataProcessing as idp
from plateypus.dataprocessing import cytokineDataProcessing as cydp
from plateypus.dataprocessing import cellDataProcessing as cdp
from plateypus.dataprocessing import proliferationDataProcessing as pdp
from plateypus.dataprocessing import killingDataProcessing as kdp
from plateypus.dataprocessing import singleCellDataProcessing as scdp
#import automatedCBAProcessingGUI as autoCBA


secondPath = '../../outputData'

concUnit = 1e9
unitPrefixDictionary = {1e12:'pM',1e9:'nM',1e6:'uM',1e3:'mM',1e0:'M'}
concUnitPrefix = unitPrefixDictionary[concUnit]

class DataProcessingStartPage(tk.Frame):
    def __init__(self, master,folderName,expNum,ex_data,bPage):
        tk.Frame.__init__(self, master)
        
        #os.chdir(master.homedirectory+'/'+folderName)
        backPage = bPage

        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l2 = tk.Label(mainWindow, text="""Datatype: """,padx = 20).grid(row=0,column=0,sticky=tk.W)
        l2a = tk.Label(mainWindow, text="Cytokine:",padx = 20).grid(row=1,column=0,sticky=tk.W)
        l2b = tk.Label(mainWindow,text="Cell:",padx = 20).grid(row=2,column=0,sticky=tk.W)
        l2c = tk.Label(mainWindow,text="Proliferation:",padx = 20).grid(row=3,column=0,sticky=tk.W)
        l2e = tk.Label(mainWindow,text="Killing:",padx = 20).grid(row=4,column=0,sticky=tk.W)
        l2d = tk.Label(mainWindow,text="Single Cell:",padx = 20).grid(row=5,column=0,sticky=tk.W)
        
        def createDataFrame(dataType):
            dataProcessingMaster(folderName,expNum,dataType,ex_data,v3.get())
        
        l3 = tk.Label(mainWindow, text="""Action: """).grid(row=0,column=1,sticky=tk.W)
        
        cytCalibrationParametersButton = tk.Button(mainWindow,text='Enter CBA bead calibration parameters',command=lambda: master.switch_frame(cydp.CalibrationParameterPage,folderName,expNum,ex_data,DataProcessingStartPage,bPage))
        cytCalibrationParametersButton.grid(row=1,column=1,sticky=tk.W)
        #cytCalibrationParametersButton = tk.Button(mainWindow,text='Create CBA gates',command=lambda: master.switch_frame(autoCBA.AutomateCBAStartPage,folderName,expNum,ex_data,DataProcessingStartPage,bPage))
        #cytCalibrationParametersButton.grid(row=1,column=2,sticky=tk.W)
        cytDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('cyt'))
        cytDfButton.grid(row=1,column=2,sticky=tk.W)

        cellDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('cell'))
        cellDfButton.grid(row=2,column=2,sticky=tk.W)
        #cellAbPanelButton = tk.Button(mainWindow,text='Edit antibody panel',command=lambda: master.switch_frame(cdp.MarkerNumberPage,folderName,expNum,ex_data,DataProcessingStartPage,bPage))
        #cellAbPanelButton.grid(row=2,column=2,sticky=tk.W)
        
        prolifGenerationGatesButton = tk.Button(mainWindow,text='Edit generation gates',command=lambda: master.switch_frame(pdp.GatingPage,folderName,expNum,ex_data,DataProcessingStartPage,bPage))
        prolifGenerationGatesButton.grid(row=3,column=1,sticky=tk.W)
        prolifDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('prolif'))
        prolifDfButton.grid(row=3,column=2,sticky=tk.W)
        
        killingDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('killing'))
        killingDfButton.grid(row=4,column=2,sticky=tk.W)

        #completeSingleCellDfButton = tk.Button(mainWindow,text='Create complete dataframes')
        #completeSingleCellDfButton.grid(row=4,column=2,sticky=tk.W)
        singleCellDfButton = tk.Button(mainWindow,text='Create dataframe',command=lambda: createDataFrame('singlecell'))
        singleCellDfButton.grid(row=5,column=2,sticky=tk.W)
        cbWindow = tk.Frame(mainWindow)
        cbWindow.grid(row=5,column=1,sticky=tk.W)
        l3 = tk.Label(cbWindow,text='Use empty wells?').grid(row=0,column=0,sticky=tk.W)
        v3 = tk.BooleanVar(value=False)
        cb = tk.Checkbutton(cbWindow, variable=v3)
        cb.grid(row=0,column=1,sticky=tk.W)

        for i,button in enumerate([cytDfButton,cellDfButton,prolifDfButton,prolifGenerationGatesButton,singleCellDfButton,killingDfButton]):
            if i == 0:
                requiredFiles = ['CBAcalibrationParameters-'+folderName+'.json']
            elif i == 2:
                requiredFiles = ['singleCellDataFrame-proliferation-'+folderName+'.pkl']
            elif i == 3:
                requiredFiles = ['logicleProliferationDf.pkl','rawProliferationDf.pkl']
            else:
                requiredFiles = []
            for requiredFile in requiredFiles:
                if requiredFile not in os.listdir('misc')+os.listdir('inputData/bulkCSVFiles')+os.listdir('outputData/pickleFiles'):
                    button.config(state=tk.DISABLED)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        #tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

def dataProcessingMaster(folderName,expNum,dataType,ex_data,useBlankWells):
    if dataType == 'singlecell' or dataType == 'prolif':
        parameterExtension = 'cell'
    else:
        parameterExtension = dataType
    experimentParameters = json.load(open('misc/experimentParameters-'+folderName+'-'+parameterExtension+'.json','r'))
    if experimentParameters['format'] == 'plate':
        experimentFormat = 'plate'
        experimentLevelLayoutDict = pickle.load(open('misc/layoutDict-'+folderName+'-'+parameterExtension+'.pkl','rb'))
    else:
        experimentFormat = 'tube'
        experimentLevelLayoutDict = pickle.load(open('misc/tubeLayout-'+folderName+'-'+parameterExtension+'.pkl','rb'))
    #experimentLevelLayoutDict = idp.tilePlateLayouts(experimentParameters,levelLayouts)
    if(dataType == 'cyt'):
        calibrationParameters = json.load(open('misc/CBAcalibrationParameters-'+folderName+'.json','r'))
        numberOfCalibrationSamples = calibrationParameters['Number']
        initialStandardVolume = calibrationParameters['Volume']
        cydp.calibrateExperiment(folderName,secondPath,concUnit,concUnitPrefix,numberOfCalibrationSamples,initialStandardVolume)
        basecytdf = idp.createBaseDataFrame(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict)
        cytdf = cydp.createCytokineDataFrame(folderName,basecytdf,concUnitPrefix)
        idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,cytdf,ex_data) 
    elif(dataType == 'cell'):
        celldf = idp.createBaseDataFrame(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict)
        idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,celldf,ex_data) 
    elif(dataType == 'prolif'):
        prolifdf = pdp.generateBulkProliferationStatistics(folderName,expNum)
        idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,prolifdf,ex_data) 
    elif(dataType == 'killing'):
        killingdf = kdp.generateBulkKillingStatistics(experimentParameters,folderName,expNum,dataType,experimentLevelLayoutDict)
        idp.saveFinalDataFrames(folderName,secondPath,expNum,dataType,killingdf,ex_data) 
        
        #Re-add parsed time values back into experiment parameter file
        #levelLabelDict = experimentParameters['levelLabelDict']
        #levelLabelDict['Time'] = list(killingDf.columns) 
        #experimentParameters['levelLabelDict'] = levelLabelDict
        #with open('misc/experimentParameters-'+folderName+'-'+dataType+'.json', 'w') as fp:
        #    json.dump(experimentParameters, fp)

    elif(dataType == 'singlecell'):
        dataType = 'singlecell'
        if experimentFormat == 'plate':
            scdp.createPlateSingleCellDataFrame(folderName,experimentParameters,experimentLevelLayoutDict,useBlankWells)
        else:
            scdf = scdp.createTubeSingleCellDataFrame(folderName,experimentParameters,experimentLevelLayoutDict)
    niceDatatypeNameDict = {'cyt':'Cytokine','cell':'Bulk cell','prolif':'Proliferation','singlecell':'Single cell','killing':'Killing'}
    tk.messagebox.showinfo("Dataframe Created", niceDatatypeNameDict[dataType]+" dataframe created!")



#!/usr/bin/env python3
import pickle,os,json
import pandas as pd
import tkinter as tk
import tkinter.ttk
import plateypus.dataprocessing.initialDataProcessing as idp
import plateypus.dataprocessing.miscFunctions as mf
import plateypus.plotting.facetPlotLibrary as fpl 
import plateypus.plotting.interactiveGUIElements as ipe

expParamDict = {'cyt':'cyt','cell':'cell','prolif':'cell','singlecell':'cell','killing':'killing'}

#Get level names and values into an easily accessible dictionary
def createLabelDict(df):
    fulldf = df.stack()
    labelDict = {}
    for i in range(fulldf.index.nlevels):
        levelName = fulldf.index.levels[i].name
        if levelName not in ['Event','event']:
            labelDict[levelName] = list(pd.unique(fulldf.index.get_level_values(levelName)))
    return labelDict

def createLabelDictWithExperimentParameters(df,experimentParameters):
    fulldf = df.stack()
    labelDict = {}
    for i in range(fulldf.index.nlevels):
        levelName = fulldf.index.levels[i].name
        if levelName in ['Event','event']:
            pass
        else:
            if 'allLevelValues' in list(experimentParameters.keys()):
                experimentParameters['levelLabelDict'] = experimentParameters['allLevelValues']
            if levelName in experimentParameters['levelLabelDict'].keys():
                if len(experimentParameters['levelLabelDict'][levelName]) > 0:
                    labelDict[levelName] = experimentParameters['levelLabelDict'][levelName]
                else:
                    labelDict[levelName] = list(pd.unique(fulldf.index.get_level_values(levelName)))
            else:
                labelDict[levelName] = list(pd.unique(fulldf.index.get_level_values(levelName)))
    return labelDict

class checkUncheckAllButton(tk.Button):
    def __init__(self,parent,checkButtonList,**kwargs):
        tk.Button.__init__(self,parent,**kwargs)
        self.checkButtonList = checkButtonList
        self.parent = parent

    def checkAll(self):
        for checkButton in self.checkButtonList:
            checkButton.select()
    
    def uncheckAll(self):
        for checkButton in self.checkButtonList:
            checkButton.deselect()

class PlotExperimentWindow(tk.Frame):
    def __init__(self, master,fName,sPage):
        with open('misc/normalPlottingBool.pkl','wb') as f:
            pickle.dump(True,f)

        global folderName,switchPage
            
        folderName = fName
        switchPage = sPage

        tk.Frame.__init__(self, master)
        
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l2 = tk.Label(mainWindow, text="""Datatype: """)
        v2 = tk.StringVar(value='cyt')
        rb2a = tk.Radiobutton(mainWindow, text="Cytokine",padx = 20, variable=v2, value='cyt')
        rb2b = tk.Radiobutton(mainWindow,text="Cell",padx = 20, variable=v2, value='cell')
        rb2c = tk.Radiobutton(mainWindow,text="Proliferation",padx = 20, variable=v2, value='prolif')
        rb2d = tk.Radiobutton(mainWindow,text="Single Cell",padx = 20, variable=v2, value='singlecell')
        rb2e = tk.Radiobutton(mainWindow,text="Killing",padx = 20, variable=v2, value='killing')
        
        l2.grid(row=0,column=0)
        rb2a.grid(row=1,column=0,sticky=tk.W)
        rb2b.grid(row=2,column=0,sticky=tk.W)
        rb2c.grid(row=3,column=0,sticky=tk.W)
        rb2d.grid(row=4,column=0,sticky=tk.W)
        rb2e.grid(row=5,column=0,sticky=tk.W)
        
        def collectInputs():
            global useModifiedDf
            modified = False 
            useModifiedDf = modified
            if modified:
                modifiedString = '-modified'
            else:
                modifiedString = ''
            global dataType
            dataType = str(v2.get())
            global experimentDf
            if dataType != 'singlecell':
                experimentDf = pd.read_pickle('outputData/pickleFiles/'+idp.dataTypeDataFrameFileNames[dataType]+'-'+folderName+modifiedString+'.pkl')
            else:
                if 'initialSingleCellDf-channel-'+folderName+modifiedString+'.pkl' in os.listdir('outputData/pickleFiles/'):
                    experimentDf = pickle.load(open('outputData/pickleFiles/'+'initialSingleCellDf-channel-'+folderName+modifiedString+'.pkl','rb'))
                else:
                    experimentDf = pd.read_hdf('outputData/pickleFiles/'+'initialSingleCellDf-channel-'+folderName+modifiedString+'.h5', 'df')
                if 'CellType' != experimentDf.index.names[0]:
                    experimentDf = pd.concat([experimentDf], keys=['TCells'], names=['CellType'])
            global trueLabelDict
            trueLabelDict = {}
            if experimentDf.columns.name == None:
                experimentDf.columns.name = 'Marker'
            experimentParametersBool = False
            for fn in os.listdir('misc'):
                if 'experimentParameters' in fn:
                    if expParamDict[dataType] in fn:
                        experimentParametersBool = True
                        experimentParameters = json.load(open('misc/experimentParameters-'+folderName+'-'+expParamDict[dataType]+'.json','r'))
            if experimentParametersBool:
                trueLabelDict = createLabelDictWithExperimentParameters(experimentDf,experimentParameters)
            else:
                trueLabelDict = createLabelDict(experimentDf)
            master.switch_frame(PlotTypePage)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(switchPage,folderName)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)


class PlotTypePage(tk.Frame):
    def __init__(self, master):

        if 'normalPlottingBool.pkl' in os.listdir('misc'):
            normalPlottingBool = pickle.load(open('misc/normalPlottingBool.pkl','rb'))
        else:
            normalPlottingBool = True

        if not normalPlottingBool:
            global useModifiedDf,dataType,experimentDf,trueLabelDict,folderName,switchPage
            plottingParams = pickle.load(open('misc/plottingParams.pkl','rb'))
            useModifiedDf = False
            dataType = 'cell'
            experimentDf = plottingParams['df']
            trueLabelDict = createLabelDict(experimentDf)
            folderName = plottingParams['folderName']
            switchPage = 'a' 

        plottableFigureDict = {'1d':['histogram','kde'],'categorical':['bar','violin','box','point','swarm','strip'],'2d':['line','scatter'],'3d':['heatmap']}
        
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
        
        l1 = tk.Label(mainWindow, text="What type of figure do you want to plot?",pady=10).grid(row=0,column = 0,columnspan = len(plottableFigureDict),sticky=tk.N)
        
        #global trueLabelDict
        #trueLabelDict = {}
        #trueLabelDict = createLabelDict(experimentDf)
         
        plotTypeRadioButtons = []
        plotSelectionString = tk.StringVar(value='1d/histogram')
        maxNumSubplots = 0
        for plotTypeIndex,plotTypeTitle in enumerate(plottableFigureDict):
            plotTypeTitleLabel = tk.Label(mainWindow,text=plotTypeTitle).grid(row=1,column=plotTypeIndex,sticky=tk.NW)
            temprblist = []
            tempselectionstring = []
            for subPlotTypeIndex,subPlotTitle in enumerate(plottableFigureDict[plotTypeTitle]):
                rb = tk.Radiobutton(mainWindow, text=subPlotTitle,padx = 20, variable=plotSelectionString, value=plotTypeTitle+'/'+subPlotTitle)
                rb.grid(row=subPlotTypeIndex+2,column=plotTypeIndex,sticky=tk.NW)
                temprblist.append(rb)
            plotTypeRadioButtons.append(temprblist)
            if len(plottableFigureDict[plotTypeTitle]) > maxNumSubplots:
                maxNumSubplots = len(plottableFigureDict[plotTypeTitle])
        
        stripSwarmBool = tk.BooleanVar()
        cb = tk.Checkbutton(mainWindow,text='Add strip/swarm points to categorical plot',variable=stripSwarmBool,pady=20)
        cb.grid(row=maxNumSubplots+2,column=0,columnspan=len(plottableFigureDict))
        
        def collectInputs():
            global plotType
            global subPlotType
            global addDistributionPoints
            addDistributionPoints = stripSwarmBool.get()
            plotType,subPlotType = plotSelectionString.get().split('/')
            master.switch_frame(selectLevelsPage,'a','b','c','d','e')
        
        def backCommand():
            #os.chdir('../../')
            if normalPlottingBool:
                master.switch_frame(PlotExperimentWindow,folderName,switchPage)
            else:
                master.switch_frame(plottingParams['homepage'],folderName,plottingParams['bp'],plottingParams['shp'])
            #master.switch_frame(PlotExperimentWindow,switchPage)
       
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP)
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Back",command=lambda: backCommand()).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)

class selectLevelsPage(tk.Frame):
    def __init__(self, master,fsp,fName,backPage,pt,shp):
        tk.Frame.__init__(self, master)
        labelWindow = tk.Frame(self)
        labelWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        global figureLevelList,fullFigureLevelBooleanList
        fullFigureLevelBooleanList = []
        figureLevelList = []
        
        l1 = tk.Label(labelWindow, text="""Which levels names do you want to be included within this figure??:""").pack()
        mainWindow = tk.Frame(self)
        levelNameCheckButtons = []
        checkButtonVariableList = []
        for levelName,i in zip(trueLabelDict.keys(),range(len(trueLabelDict.keys()))):
            includeLevelBool = tk.BooleanVar(value=True)
            cb = tk.Checkbutton(mainWindow, text=levelName,padx = 20, variable=includeLevelBool,onvalue=True)
            cb.grid(row=i+3,column=1,sticky=tk.W)
            cb.select()
            levelNameCheckButtons.append(cb)
            checkButtonVariableList.append(includeLevelBool)
        
        checkButtonWindow = tk.Frame(self)
        checkAllButton1 = checkUncheckAllButton(checkButtonWindow,levelNameCheckButtons, text='Check All')
        checkAllButton1.configure(command=checkAllButton1.checkAll)
        checkAllButton1.pack(side=tk.LEFT)
        
        uncheckAllButton1 = checkUncheckAllButton(checkButtonWindow,levelNameCheckButtons, text='Uncheck All')
        uncheckAllButton1.configure(command=checkAllButton1.uncheckAll)
        uncheckAllButton1.pack(side=tk.LEFT)
        
        checkButtonWindow.pack(side=tk.TOP)
        mainWindow.pack(side=tk.TOP,padx=10)

        def collectInputs():
            includeLevelList = []
            for checkButtonVariable in checkButtonVariableList:
                includeLevelList.append(checkButtonVariable.get())
            for figureLevelBool,levelName in zip(includeLevelList,trueLabelDict):
                if figureLevelBool:
                    figureLevelList.append(levelName)
                    fullFigureLevelBooleanList.append(True)
                else:
                    fullFigureLevelBooleanList.append(False)
            master.switch_frame(selectLevelValuesPage,assignLevelsToParametersPage,trueLabelDict,selectLevelsPage,selectLevelsPage,fsp,fName,shp,'a')
        
        def quitCommand():
            exitBoolean = True
            quit()

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).pack(in_=buttonWindow,side=tk.LEFT)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(PlotTypePage)).pack(in_=buttonWindow,side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quitCommand()).pack(in_=buttonWindow,side=tk.LEFT)

class selectLevelValuesPage(tk.Frame):
    #(selectLevelValuesPage,SelectDimensionsPage,trueLabelDict,DataSelectionHomePage,finalSwitchPage,folderName,secondaryhomepage,processType)
    #master.switch_frame(selectLevelValuesPage,SelectDimensionsPage,trueLabelDict,DataSelectionHomePage,backpage,finalSwitchPage,folderName,secondaryhomepage,processType)
    def __init__(self, master,switchPage,trueLabelDict,backPage,secondaryBackPage,fsp,fName,shp,pt):
        tk.Frame.__init__(self, master)
        
        includeLevelValueList = []
        
        labelWindow = tk.Frame(self)
        labelWindow.pack(side=tk.TOP,padx=10,fill=tk.X,expand=True)
        
        l1 = tk.Label(labelWindow, text='Which specific level values do you want to include in the figure?',pady=10).grid(row=0,column = 0,columnspan=len(trueLabelDict)*6)
        levelValueCheckButtonList = []
        overallCheckButtonVariableList = []
        checkAllButtonList = []
        uncheckAllButtonList = []
        i=0
        maxNumLevelValues = 0
        for levelName in trueLabelDict:
            if len(trueLabelDict[levelName]) > maxNumLevelValues:
                maxNumLevelValues = len(trueLabelDict[levelName])
        """BEGIN TEMP SCROLLBAR CODE"""
        labelWindow1 = tk.Frame(self)
        labelWindow1.pack(side=tk.TOP,padx=10,fill=tk.X,expand=True)
        
        #Make canvas
        w1 = tk.Canvas(labelWindow1, width=1200, height=400,background="white", scrollregion=(0,0,2000,33*maxNumLevelValues))

        #Make scrollbar
        scr_v1 = tk.Scrollbar(labelWindow1,orient=tk.VERTICAL)
        scr_v1.pack(side=tk.RIGHT,fill=tk.Y)
        scr_v1.config(command=w1.yview)
        #Add scrollbar to canvas
        w1.config(yscrollcommand=scr_v1.set)
        
        scr_v2 = tk.Scrollbar(labelWindow1,orient=tk.HORIZONTAL)
        scr_v2.pack(side=tk.BOTTOM,fill=tk.X)
        scr_v2.config(command=w1.xview)
        w1.config(xscrollcommand=scr_v2.set)
        w1.pack(fill=tk.BOTH,expand=True)
        #Make and add frame for widgets inside of canvas
        #canvas_frame = tk.Frame(w1)
        labelWindow = tk.Frame(w1)
        labelWindow.pack()
        w1.create_window((0,0),window=labelWindow, anchor = tk.NW)
        """END TEMP SCROLLBAR CODE"""
        for levelName in trueLabelDict:
            j=0
            levelCheckButtonList = []
            levelCheckButtonVariableList = []
            levelLabel = tk.Label(labelWindow, text=levelName+':')
            levelLabel.grid(row=1,column = i*6,sticky=tk.N,columnspan=5)
            for levelValue in trueLabelDict[levelName]:
                includeLevelValueBool = tk.BooleanVar()
                cb = tk.Checkbutton(labelWindow, text=levelValue, variable=includeLevelValueBool)
                cb.grid(row=j+4,column=i*6+2,columnspan=2,sticky=tk.W)
                labelWindow.grid_columnconfigure(i*6+3,weight=1)
                cb.select()
                levelCheckButtonList.append(cb)
                levelCheckButtonVariableList.append(includeLevelValueBool)
                j+=1
            
            checkAllButton1 = checkUncheckAllButton(labelWindow,levelCheckButtonList, text='Check All')
            checkAllButton1.configure(command=checkAllButton1.checkAll)
            checkAllButton1.grid(row=2,column=i*6,sticky=tk.N,columnspan=3)
            checkAllButtonList.append(checkAllButton1)
            
            uncheckAllButton1 = checkUncheckAllButton(labelWindow,levelCheckButtonList, text='Uncheck All')
            uncheckAllButton1.configure(command=checkAllButton1.uncheckAll)
            uncheckAllButton1.grid(row=2,column=i*6+3,sticky=tk.N,columnspan=3)
            uncheckAllButtonList.append(checkAllButton1)

            levelValueCheckButtonList.append(levelCheckButtonList)
            overallCheckButtonVariableList.append(levelCheckButtonVariableList)
            i+=1

        def collectInputs():
            for checkButtonVariableList in overallCheckButtonVariableList:
                tempLevelValueList = []
                for checkButtonVariable in checkButtonVariableList:
                    tempLevelValueList.append(checkButtonVariable.get())
                includeLevelValueList.append(tempLevelValueList)
            #master.switch_frame(assignLevelsToParametersPage)
            master.switch_frame(switchPage,includeLevelValueList)
        
        def quitCommand():
            exitBoolean = True
            quit()
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=maxNumLevelValues+4,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backPage,fsp,fName,secondaryBackPage,pt,shp)).grid(row=maxNumLevelValues+4,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quitCommand()).grid(row=maxNumLevelValues+4,column=2)

class assignLevelsToParametersPage(tk.Frame):
    
    def __init__(self, master,temp):
        parameterTypeDict = {
                'categorical':['Color','Order', 'Row', 'Column','None'],
                '1d':['Color','Row','Column','None'],
                '2d':['Marker','Color','Size','Row','Column','X Axis Values','None'],
                '3d':['Row','Column','X Axis Values','Y Axis Values']}
        
        tk.Frame.__init__(self, master)
        global includeLevelValueList
        includeLevelValueList = temp
        global parametersSelected
        parametersSelected = {}
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        
        l1 = tk.Label(mainWindow, text='Which plotting parameter do you want to assign to each of your figure levels?',pady=10).grid(row=0,column = 0,columnspan = len(figureLevelList))
        rblist = []
        parameterVarList = []
        for figureLevel,figureLevelIndex in zip(figureLevelList,range(len(figureLevelList))):
            v = tk.IntVar()
            temprblist = []
            levelLabel = tk.Label(mainWindow, text=figureLevel+':')
            levelLabel.grid(row=1,column=figureLevelIndex,sticky=tk.NW)
            for plottingParameter,parameterIndex in zip(parameterTypeDict[plotType],range(len(parameterTypeDict[plotType]))):
                rb = tk.Radiobutton(mainWindow, text=plottingParameter,padx = 20, variable=v, value=parameterIndex)
                rb.grid(row=parameterIndex+2,column=figureLevelIndex,sticky=tk.NW)
                temprblist.append(rb)
            rblist.append(temprblist)
            parameterVarList.append(v)
        
        def collectInputs():
            for parameterVar,levelName in zip(parameterVarList,figureLevelList):
                if parameterTypeDict[plotType][parameterVar.get()] not in parametersSelected.keys():
                    parametersSelected[parameterTypeDict[plotType][parameterVar.get()]] = levelName
                else:
                    if not isinstance(parametersSelected[parameterTypeDict[plotType][parameterVar.get()]],list):
                        parametersSelected[parameterTypeDict[plotType][parameterVar.get()]] = [parametersSelected[parameterTypeDict[plotType][parameterVar.get()]]]+[levelName]
                    else:
                        parametersSelected[parameterTypeDict[plotType][parameterVar.get()]].append(levelName)
            master.switch_frame(plotElementsGUIPage)
        
        def quitCommand():
            exitBoolean = True
            quit()
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=len(parameterTypeDict[plotType])+2,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(selectLevelValuesPage,assignLevelsToParametersPage,trueLabelDict,selectLevelsPage,'a','b','c','d','e')).grid(row=len(parameterTypeDict[plotType])+2,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quitCommand()).grid(row=len(parameterTypeDict[plotType])+2,column=2)

def getDefaultAxisTitles():
        xaxistitle = ''
        yaxistitle = ''
        cbartitle = ''
        cytokineDefault = 'Concentration (nM)'
        if plotType == '1d':
            xaxistitle = 'MFI'
            yaxistitle = 'Frequency'
        else:
            if plotType == 'categorical':
                xaxistitle = parametersSelected['Order']
                if dataType == 'cyt':
                    yaxistitle = cytokineDefault
            else:
                xaxistitle = parametersSelected['X Axis Values']
                if dataType == 'cyt':
                    if plotType == '2d':
                        yaxistitle = cytokineDefault
                    else:
                        cbartitle = cytokineDefault
        if xaxistitle == 'Time':
            xaxistitle += ' (hours)'
        return [xaxistitle,yaxistitle,cbartitle]

class plotElementsGUIPage(tk.Frame):
    def __init__(self, master):
        if 'experimentParameters-'+folderName+'-'+expParamDict[dataType]+'.json' in os.listdir('misc'):
            experimentParameters = json.load(open('misc/experimentParameters-'+folderName+'-'+expParamDict[dataType]+'.json','r'))
        else:
            tempDict = {}
            stackedDf = experimentDf.stack()
            for level in stackedDf.index.names:
                levelValues = list(pd.unique(stackedDf.index.get_level_values(level)))
                tempDict[level] = levelValues
            experimentParameters = {}
            experimentParameters['levelLabelDict'] = tempDict
        """ 
        global dataType
        if 'cell' in pickleFileName: 
            dataType = 'cell'
        elif 'cyt' in pickleFileName:
            dataType = 'cyt'
        elif 'prolif' in pickleFileName:
            dataType = 'prolif'
        else:
            dataType = ''
        """
        self.tld = trueLabelDict

        axisDict = {'categorical':['X','Y'],'1d':['Y'],'2d':['X','Y'],'3d':['X','Y','Colorbar']}
        scalingList = ['Linear','Logarithmic','Biexponential']
        axisSharingList = ['col','row','']
        axisTitleDefaults = getDefaultAxisTitles() 
        
        tk.Frame.__init__(self, master)
        
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)

        tk.Label(mainWindow, text='Title: ').grid(row=1,column=0,sticky=tk.W)
        for scaling,scalingIndex in zip(scalingList,range(len(scalingList))):
            tk.Label(mainWindow, text=scaling+' Scaling: ').grid(row=scalingIndex+2,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Linear Range (Biexponential Scaling): ').grid(row=len(scalingList)+2,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Convert to numeric: ').grid(row=len(scalingList)+3,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Share axis across: ').grid(row=len(scalingList)+4,column=0,sticky=tk.W)
        tk.Label(mainWindow, text='Axis limits: ').grid(row=len(scalingList)+5,column=0,sticky=tk.W)

        entryList = []
        scalingVariableList = []
        radioButtonList = []
        checkButtonList = []
        checkButtonVarList = []
        radioButtonList2 = []
        radioButtonVarList2 = []
        linearRangeScalingList = []
        limitEntryList = []
        for axis,axisIndex in zip(axisDict[plotType],range(len(axisDict[plotType]))):
            tk.Label(mainWindow, text=axis+ ' Axis').grid(row=0,column=axisIndex+1)
            
            e1 = tk.Entry(mainWindow)
            e1.grid(row=1,column=axisIndex+1)
            e1.insert(0, axisTitleDefaults[axisIndex])
            entryList.append(e1)
            
            axisRadioButtonList = []
            v = tk.StringVar(value='Linear')
            for scaling,scalingIndex in zip(scalingList,range(len(scalingList))):
                rb = tk.Radiobutton(mainWindow,variable=v,value=scaling)
                rb.grid(row=scalingIndex+2,column=axisIndex+1)
                axisRadioButtonList.append(rb)
            radioButtonList.append(axisRadioButtonList)
            scalingVariableList.append(v)
            
            e2 = tk.Entry(mainWindow)
            e2.grid(row=len(scalingList)+2,column=axisIndex+1)
            linearRangeScalingList.append(e2)
            
            b = tk.BooleanVar(value=False)
            cb = tk.Checkbutton(mainWindow,variable=b)
            cb.grid(row=len(scalingList)+3,column=axisIndex+1)
            checkButtonList.append(cb)
            checkButtonVarList.append(b)
       
            shareWindow = tk.Frame(mainWindow)
            shareWindow.grid(row=len(scalingList)+4,column=axisIndex+1)
            shareString = tk.StringVar(value='None')
            rb2a = tk.Radiobutton(shareWindow,variable=shareString,text='All',value='all')
            rb2b = tk.Radiobutton(shareWindow,variable=shareString,text='Row',value='row')
            rb2c = tk.Radiobutton(shareWindow,variable=shareString,text='Col',value='col')
            rb2d = tk.Radiobutton(shareWindow,variable=shareString,text='None',value='none')
            shareString.set('all')
            rb2a.grid(row=0,column=1)
            rb2b.grid(row=0,column=2)
            rb2c.grid(row=0,column=3)
            rb2d.grid(row=0,column=4)
            radioButtonList2.append([rb2a,rb2b])
            radioButtonVarList2.append(shareString)

            limitWindow = tk.Frame(mainWindow)
            limitWindow.grid(row=len(scalingList)+5,column=axisIndex+1)
            #ll = tk.Label(limitWindow,text='Lower:').grid(row=0,column=0)
            e3 = tk.Entry(limitWindow,width=5)
            e3.grid(row=0,column=1)
            #ul = tk.Label(limitWindow,text='Upper:').grid(row=0,column=2)
            e4 = tk.Entry(limitWindow,width=5)
            e4.grid(row=0,column=3)
            limitEntryList.append([e3,e4])

        def collectInputs(plotAllVar):
            plotOptions = {}
            for axis,axisIndex in zip(axisDict[plotType],range(len(axisDict[plotType]))):
                share = radioButtonVarList2[axisIndex].get()
                if share == 'none':
                    share = False
                plotOptions[axis] = {'axisTitle':entryList[axisIndex].get(),
                        'axisScaling':scalingVariableList[axisIndex].get(),
                        'linThreshold':linearRangeScalingList[axisIndex].get(),
                        'numeric':checkButtonVarList[axisIndex].get(),
                        'share':share,
                        'limit':[limitEntryList[axisIndex][0].get(),limitEntryList[axisIndex][1].get()]}
            
            plotSpecificDict = {}
            if subPlotType == 'kde':
                scaleBool = ipe.getRadiobuttonValues(modeScaleRadiobuttonVarsDict)['scale to mode']
                if scaleBool == 'yes':
                    plotSpecificDict['scaleToMode'] = True
                else:
                    plotSpecificDict['scaleToMode'] = False
                plotSpecificDict['smoothing'] = int(ipe.getSliderValues(smoothingSliderList,['smoothing'])['smoothing'])
            
            useModifiedDf = False
            sName = titleEntry.get()
            subsettedDfList,subsettedDfListTitles,figureLevels,levelValuesPlottedIndividually = fpl.produceSubsettedDataFrames(experimentDf.stack().to_frame('temp'),fullFigureLevelBooleanList,includeLevelValueList,self.tld)
            fpl.plotFacetedFigures(folderName,plotType,subPlotType,dataType,subsettedDfList,subsettedDfListTitles,figureLevels,levelValuesPlottedIndividually,useModifiedDf,experimentDf,plotOptions,parametersSelected,addDistributionPoints,originalLevelValueOrders=experimentParameters['levelLabelDict'],subfolderName=sName,context=ipe.getRadiobuttonValues(contextRadiobuttonVarsDict)['context'],height=float(heightEntry.get()),aspect=float(widthEntry.get()),titleBool=ipe.getRadiobuttonValues(plotTitleRadiobuttonVarsDict)['plotTitle'],colwrap=int(colWrapEntry.get()),legendBool=ipe.getRadiobuttonValues(legendRadiobuttonVarsDict)['legend'],cmap=cmapEntry.get(),plotAllVar=plotAllVar,titleAdjust=titleAdjustEntry.get(),plotSpecificDict = plotSpecificDict)
            
        titleWindow = tk.Frame(self)
        titleWindow.pack(side=tk.TOP,pady=10)
        tk.Label(titleWindow,text='Enter subfolder for these plots (optional):').grid(row=0,column=0)
        titleEntry = tk.Entry(titleWindow,width=15)
        titleEntry.grid(row=0,column=1)
        
        miscOptionsWindow = tk.Frame(self)
        miscOptionsWindow.pack(side=tk.TOP,pady=10)
        
        contextWindow = tk.Frame(miscOptionsWindow)
        contextWindow.grid(row=0,column=0,sticky=tk.N)
        contextRadiobuttonList,contextRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(contextWindow,['context'],{'context':['notebook','talk','poster']}) 
        
        figureDimensionWindow = tk.Frame(miscOptionsWindow)
        figureDimensionWindow.grid(row=0,column=1,sticky=tk.N)
        tk.Label(figureDimensionWindow,text='figure dimensions').grid(row=0,column=0)
        tk.Label(figureDimensionWindow,text='height:').grid(row=1,column=0)
        tk.Label(figureDimensionWindow,text='width:').grid(row=2,column=0)
        heightEntry = tk.Entry(figureDimensionWindow,width=3)
        if plotType != '1d':
            heightEntry.insert(0, '5')
        else:
            heightEntry.insert(0, '3')
        widthEntry = tk.Entry(figureDimensionWindow,width=3)
        widthEntry.insert(0, '1')
        heightEntry.grid(row=1,column=1)
        widthEntry.grid(row=2,column=1)
        
        plotTitleWindow = tk.Frame(miscOptionsWindow)
        plotTitleWindow.grid(row=0,column=2,sticky=tk.N)
        plotTitleRadiobuttonList,plotTitleRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(plotTitleWindow,['plotTitle'],{'plotTitle':['yes','no']}) 
        
        legendWindow = tk.Frame(miscOptionsWindow)
        legendWindow.grid(row=0,column=3,sticky=tk.N)
        legendRadiobuttonList,legendRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(legendWindow,['legend'],{'legend':['yes','no']}) 
        
        colWrapWindow = tk.Frame(miscOptionsWindow)
        colWrapWindow.grid(row=0,column=4,sticky=tk.N)
        tk.Label(colWrapWindow,text='column wrap:').grid(row=0,column=0)
        colWrapEntry = tk.Entry(colWrapWindow,width=5)
        colWrapEntry.insert(0, '5')
        colWrapEntry.grid(row=1,column=0)
        
        titleAdjustWindow = tk.Frame(miscOptionsWindow)
        titleAdjustWindow.grid(row=0,column=5,sticky=tk.N)
        tk.Label(titleAdjustWindow,text='title location (% of window):').grid(row=0,column=0)
        titleAdjustEntry = tk.Entry(titleAdjustWindow,width=5)
        titleAdjustEntry.insert(0, '')
        titleAdjustEntry.grid(row=1,column=0)
        
        #outlierWindow = tk.Frame(miscOptionsWindow)
        #outlierWindow.grid(row=0,column=6,sticky=tk.N)
        #outlierRadiobuttonList,outlierRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(outlierWindow,['remove outliers'],{'remove outliers':['yes','no']}) 
        #outlierRadiobuttonVarsDict['remove outliers'].set('no')
        
        cmapWindow = tk.Frame(miscOptionsWindow)
        cmapWindow.grid(row=0,column=6,sticky=tk.N)
        tk.Label(cmapWindow,text='Colormap:').grid(row=0,column=0)
        cmapEntry = tk.Entry(cmapWindow,width=10)
        cmapEntry.grid(row=1,column=0)
        
        if subPlotType == 'kde':
            #Scale to mode button
            subPlotSpecificWindow = tk.Frame(self)
            subPlotSpecificWindow.pack(side=tk.TOP,pady=10)
            modeScaleWindow = tk.Frame(subPlotSpecificWindow)
            modeScaleWindow.grid(row=0,column=0,sticky=tk.N)
            modeScaleRadiobuttonList,modeScaleRadiobuttonVarsDict = ipe.createParameterSelectionRadiobuttons(modeScaleWindow,['scale to mode'],{'scale to mode':['yes','no']}) 
            modeScaleRadiobuttonVarsDict['scale to mode'].set('no')
            #Smoothness (or bandwidth) slider
            smoothnessWindow = tk.Frame(subPlotSpecificWindow)
            smoothnessWindow.grid(row=0,column=1,sticky=tk.N)
            smoothingSliderList = ipe.createParameterAdjustmentSliders(smoothnessWindow,['smoothing'],{'smoothing':[1,99,2,27]})
        
        plotButtonWindow = tk.Frame(self)
        plotButtonWindow.pack(side=tk.TOP,pady=10)
        tk.Button(plotButtonWindow, text="Generate First Plot",command=lambda: collectInputs(False)).grid(row=0,column=0)
        tk.Button(plotButtonWindow, text="Generate All Plots",command=lambda: collectInputs(True)).grid(row=0,column=1)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)
        
        def okCommand():
            master.switch_frame(PlotTypePage)
        
        tk.Button(buttonWindow, text="Finish",command=lambda: okCommand()).grid(row=len(scalingList)+4,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(assignLevelsToParametersPage,includeLevelValueList)).grid(row=len(scalingList)+4,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).grid(row=len(scalingList)+4,column=2)



#!/usr/bin/env python3
import os,subprocess,pickle,math
import tkinter as tk
import tkinter.ttk
import tkinter.font as tkfont

def setMaxWidth(stringList, element):
    f = tkfont.nametofont(element.cget("font"))
    zerowidth=f.measure("M")
    w=max([f.measure(i) for i in stringList])/zerowidth
    element.config(width=max([6,math.ceil(1.5*w)]))

class NewProjectWindow(tk.Frame):
    def __init__(self,master,backpage):
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
        
        l1 = tk.Label(mainWindow, text="Project Name: ")
        e1 = tk.Entry(mainWindow)

        l2 = tk.Label(mainWindow, text="Path to Project folder (must start with a ~ or /)")
        e2 = tk.Entry(mainWindow)
        
        l3 = tk.Label(mainWindow, text="For reference, path to current folder is: \n"+os.getcwd()+'/\n')
        
        def createProject():
            projectName = e1.get()
            rawPathName = e2.get()
            if rawPathName[-1] != '/':
                rawPathName+='/'
            if rawPathName.split('/')[-2] == projectName:
                rawPathName = rawPathName[:rawPathName[:-1].rfind('/')]+'/'
            if projectName not in os.listdir(rawPathName):
                subprocess.run(['mkdir',rawPathName+projectName])
            if 'pathDict.pkl' in os.listdir(master.homedirectory+'misc'):
                pathDict = pickle.load(open(master.homedirectory+'misc/pathDict.pkl','rb'))
            else:
                pathDict = {}
            if projectName in pathDict.keys():
                del pathDict[projectName]
            pathDict[projectName] = rawPathName
            with open(master.homedirectory+'misc/pathDict.pkl','wb') as f:
                pickle.dump(pathDict,f)
            tk.messagebox.showinfo("Project Created", "Project\n"+projectName+"\nhas been created.")

        b = tk.Button(mainWindow,text='Create new project',command=lambda:createProject())
        
        l1.grid(row=0,column=0,sticky=tk.W)
        e1.grid(row=0,column=1,sticky=tk.W)
        l2.grid(row=1,column=0,sticky=tk.W)
        e2.grid(row=1,column=1,sticky=tk.W)
        l3.grid(row=3,column=0,columnspan=2)
        b.grid(row=2,column=0,columnspan=2)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backpage)).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)

class RemoveProjectWindow(tk.Frame):
    def __init__(self,master,backpage):
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
        
        l1 = tk.Label(mainWindow, text="Project name to remove (will not remove files, only path): ")
        pathDict = pickle.load(open(master.homedirectory+'misc/pathDict.pkl','rb'))
        projects = list(pathDict.keys())
        projectVar = tk.StringVar()
        projectMenu = tk.OptionMenu(mainWindow,projectVar,*projects)
        setMaxWidth(projects,projectMenu)

        def removeProject():
            projectName = projectVar.get()
            del pathDict[projectName]
            with open(master.homedirectory+'misc/pathDict.pkl','wb') as f:
                pickle.dump(pathDict,f)
            tk.messagebox.showinfo("Project Removed", "Project\n"+projectName+"\nhas been removed.")

        b = tk.Button(mainWindow,text='REMOVE project',command=lambda:removeProject())
        
        l1.grid(row=0,column=0,sticky=tk.W)
        projectMenu.grid(row=0,column=1)
        b.grid(row=2,column=0,columnspan=2)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backpage)).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)

class NewExperimentWindow(tk.Frame):
    def __init__(self,master,backpage):
        tk.Frame.__init__(self, master)
        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10)
         
        pathDict = pickle.load(open(master.homedirectory+'misc/pathDict.pkl','rb'))
        projects = list(pathDict.keys())
        projectTitle = tk.Label(mainWindow,text='Project name: ')
        projectMenu = tkinter.ttk.Combobox(mainWindow,values = projects)
        projectMenu.width = len(max(projects,key=len))
        if len(projects) == 1:
            projectMenu.set(projectMenu['values'][0])

        l1 = tk.Label(mainWindow, text="Experiment Date (YYYYMMDD): ")
        e1 = tk.Entry(mainWindow)

        l2 = tk.Label(mainWindow, text="Experiment Name: ")
        e2 = tk.Entry(mainWindow)
        
        def createExperiment():
            experimentName = e2.get()
            amendedExperimentName = experimentName.replace('-','_')
            amendedExperimentName = amendedExperimentName.replace('/','_')
            amendedExperimentName = amendedExperimentName.replace(' ','_')
            experimentName = e1.get()+'-'+amendedExperimentName
            projectName = projectMenu.get()
            pathName = pathDict[projectName]
            subprocess.run(['mkdir',pathName+projectName+'/'+experimentName])
            subfolders = ['inputData','outputData','plots','misc']
            subsubfoldersDict = {'inputData':['fcsFiles','singleCellCSVFiles','bulkCSVFiles'],'outputData':['excelFiles','pickleFiles','analysisFiles']}
            subsubsubfoldersDict = {'analysisFiles':['scaledData','reducedData','clusteredData','subsettedData','clusterFrequencyData']}
            for subfolder in subfolders:
                subprocess.run(['mkdir',pathName+projectName+'/'+experimentName+'/'+subfolder])
                if subfolder in subsubfoldersDict.keys():
                    subsubfolders = subsubfoldersDict[subfolder]
                    for subsubfolder in subsubfolders:
                        subprocess.run(['mkdir',pathName+projectName+'/'+experimentName+'/'+subfolder+'/'+subsubfolder])
                        if subsubfolder in subsubsubfoldersDict.keys():
                            subsubsubfolders = subsubsubfoldersDict[subsubfolder]
                            for subsubsubfolder in subsubsubfolders:
                                subprocess.run(['mkdir',pathName+projectName+'/'+experimentName+'/'+subfolder+'/'+subsubfolder+'/'+subsubsubfolder])
            
            tk.messagebox.showinfo("Experiment Created", "Experiment\n"+experimentName+"\nin Project \n"+projectName+"\nhas been created.")

        b = tk.Button(mainWindow,text='Create experiment',command=lambda:createExperiment())
        
        projectTitle.grid(row=0,column=0)
        projectMenu.grid(row=0,column=1)
        l1.grid(row=1,column=0,sticky=tk.W)
        e1.grid(row=1,column=1,sticky=tk.W)
        l2.grid(row=2,column=0,sticky=tk.W)
        e2.grid(row=2,column=1,sticky=tk.W)
        b.grid(row=3,column=0,columnspan=2)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(backpage)).pack(side=tk.LEFT)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).pack(side=tk.LEFT)
